package edu.uoc.integra.business.functions;

import java.util.ArrayList;
import java.util.List;

import org.hibernate.Hibernate;
import org.hibernate.Query;
import org.hibernate.Session;
import org.hibernate.SessionFactory;
import org.hibernate.cfg.Configuration;

import edu.uoc.integra.business.exceptions.ExceptionControl;
import edu.uoc.integra.lib.utils.Validations;

/**
 * <p>Clase que implementa los m&eacute;todos comunes utilizados por las interfaces
 * del negocio</p>
 * 
 * @author Miguel Angel Pineda Cruz
 * @version 1.0 
 */
public abstract class IBCommonManagerImpl
{

	//************************************************************//
	// ATRIBUTOS.                                                 //
	//************************************************************//

	/** <p>Tablas de la base de datos.</p> */
	public enum Table {
		
		// Tablas definidas en la Base de Datos.
		CONFIG("BOConfig"),						
		USER("BOUser"),
		PROJECT("BOProject"),
		ROL("BORol"),
		STATE("BOState"),
		WORKFLOW("BOWorkFlow"),
		WORKFLOWSTATE("BOWorkFlow_State"),
		USERROL("BOUserRol"),
		INFORM("BOInform"),
		ARTEFACT("BOArtefact"),
		ARTEFACTU("BOArtefactU"),
		HISTORY("BOHistory"),
		ATTACHMENT("BOAttachment")
		;
		
		// Nombre de la tabla.
		private String tableName;
		
		// Constructores
		/** 
		 * <p>Construye un <i>Table</i> y le asigna al atributo <i>tableName</i>
		 * con el valor que recibe como par&aacute;metro.</p>
		 * 
		 * @param tableName <code>String</code> nombre de la tabla. 
		 */
		Table(String tableName)
		{
			this.tableName = tableName;
		}
		
		//M&eacute;todos
		@Override
		public String toString()
		{
			return this.tableName;
		}
	}
	
	/** <p>Tipo de Par&aacute;metros de Configuraci&oacute;n</p> */
	public enum TypeParam {
		
		// Tipos de Par&aacute;metros definidos
		USER("User"), 
		EMAIL("Email"), 
		PROJECT("Project");
		
		// Nombre del tipo de parametro
		private String typeName;
		
		// Constructores
		/** 
		 * <p>Construye un <i>TypeParam</i> y le asigna al atributo <i>typeName</i>
		 * con el valor que recibe como par&aacute;metro.</p>
		 * 
		 * @param typeName <code>String</code> tipo de par&aacute;metro 
		 */
		TypeParam(String typeName)
		{
			this.typeName = typeName;
		}

		//M&eacute;todos
		public String toString()
		{
			return this.typeName;
		}
		
	}
	
	/** <p>Tipo de Par&aacute;metros de Autorizaci&oacute;n</p> */
	public enum TypeAutoriz {
		
		// Tipo de Autorizaciones definidas
		ADMINISTRADOR("Administrador"), 
		AVANZADO("Avanzado"), 
		NORMAL("Normal");
		
		// Nombre del tipo de autorizacion
		private String typeAutoriz;
		
		// Constructores
		/** 
		 * <p>Construye un <i>TypeAutoriz</i> y le asigna al atributo <i>typeAutoriz</i>
		 * con el valor que recibe como par&aacute;metro.</p>
		 * 
		 * @param typeAutoriz <code>String</code> tipo de autorizaci&oacute;n 
		 */
		TypeAutoriz(String typeAutoriz)
		{
			this.typeAutoriz = typeAutoriz;
		}
		
		//M&eacute;todos
		public String toString()
		{
			return this.typeAutoriz;
		}
	}
	
	
	/** <p>Par&aacute;metros de Email</p> */
	public enum ParamEmail {
		
		// Par&aacute;metros definidos de email
		HOST("mail.server.host"), 
		PORT("mail.server.port"), 
		TLS("mail.server.tls.enable"),
		AUTH("mail.server.auth.enable"),
		USER("mail.server.username"),
		PASSWD("mail.server.password"),
		FROM("mail.server.from"),
		SOCKET_PORT("mail.smtp.socketFactory.port"),
		SOCKET_CLASS("mail.smtp.socketFactory.class");
		
		// Nombre del parametro de email
		private String paramName;
		
		// Constructores
		/** 
		 * <p>Construye un <i>paramEmail</i> y le asigna al atributo <i>paramName</i>
		 * con el valor que recibe como par&aacute;metro.</p>
		 * 
		 * @param paramName <code>String</code> Nombre del Par&aacute;metro 
		 */
		ParamEmail(String paramName)
		{
			this.paramName = paramName;
		}

		//M&eacute;todos
		public String toString()
		{
			return this.paramName;
		}
		
	}
	
	/** <p>SessionFactory</code> de <i>Hibernate</i>.</p> */
	protected SessionFactory sessionFactory;

	
	//************************************************************//
	// CONSTRUCTOR.                                               //
	//************************************************************//
	/**
	 * <p>Inicializa el marco de persistencia <i>Hibernate</i>.</p>
	 * 
	 * @throws ExceptionControl para el caso que se produzca una excepci&oacute;n.
	 */
	public IBCommonManagerImpl() throws ExceptionControl
	{
		super();
		
		try
		{
			// Instanciamos un nuevo objeto de tipo SessionFactory.
			sessionFactory = new Configuration().configure().buildSessionFactory();
		}
		catch (Exception ex)
		{
			// Lanzamos la exception
			throw new ExceptionControl(ex.getMessage());
		}
	}

	
	//************************************************************//
	// METODOS                                                    //
	//************************************************************//

	/**
	 * <p>Este m&eacute;todo realiza una consulta sobre la tabla <i>table</i> y obtiene el registro
	 * que contiene los campos <i>field_0, field_1</i>  con los valores de los par&aacute;metros <i>value_0, value_1</i> respectivamente.
	 * En caso de no existir o haber m&aacute;s de uno retorna nulo</p>
	 *  
	 * @param session <code>Session</code> sesi&oacute;n de Hibernate.
	 * @param table <code>Table</code> la tabla de la base de datos.
	 * @param field_0, field_1 <code>String</code> campos a comprobar.
	 * @param value_0, value_1 <code>String</code> valores a buscar.
	 * @return <code>Object</code>  contiene la informacion del objeto dentro de 
	 * la base de datos o <code>null</code> en caso de no existir.
	 */
	@SuppressWarnings("rawtypes")
	protected Object existRegister(Session session, Table table, String field_0, String value_0, String field_1, String value_1)
	{
		
		// Comprobamos que los par&aacute;metros no son null y adem&aacute;s que se ha inicializado la sesi&oacute;n
		if ((session.isOpen()) && (table != null) && (!Validations.isEmpty(field_0)) && (!Validations.isEmpty(field_1)))
		{
			// Preparamos la consulta HQL.
			Query query = session.createQuery("from " + table + " where " + field_0 + "=? and " + field_1 + "=?");
			
			// Establecemos el valor del atributo.
			query.setParameter(0, value_0, Hibernate.STRING);
			query.setParameter(1, value_1, Hibernate.STRING);
			
			// Ejecutamos la consulta.
			List results = query.list();
			
			// Comprobamos el n&uacute;mero de resultados.
			if (results.size() == 1)
			{
				// Si el n&uacute;mero de elementos es 1 existe el valor.
				return results.get(0);
			}
		}
		
		// No existe valor.
		return null;
	}

	/**
	 * <p>Este m&eacute;todo realiza una consulta sobre la tabla <i>table</i> y obtiene el registro
	 * que contiene los campos <i>field_0, field_1</i>  con los valores de los par&aacute;metros <i>value_0, value_1</i> respectivamente.
	 * En caso de no existir o haber m&aacute;s de uno retorna nulo</p>
	 *  
	 * @param session <code>Session</code> sesi&oacute;n de Hibernate.
	 * @param table <code>Table</code> la tabla de la base de datos.
	 * @param field_0, field_1 <code>String</code> campos a comprobar.
	 * @param value_0 <code>int</code> valor a buscar.
	 * @param value_1 <code>String</code> valor a buscar.
	 * @return <code>Object</code>  contiene la informacion del objeto dentro de 
	 * la base de datos o <code>null</code> en caso de no existir.
	 */
	@SuppressWarnings("rawtypes")
	protected Object existRegister(Session session, Table table, String field_0, int value_0, String field_1, String value_1)
	{
		
		// Comprobamos que los par&aacute;metros no son null y adem&aacute;s que se ha inicializado la sesi&oacute;n
		if ((session.isOpen()) && (table != null) && (!Validations.isEmpty(field_0)) && (!Validations.isEmpty(field_1)))
		{
			
			// Preparamos la consulta HQL.
			Query query = session.createQuery("from " + table + " where " + field_0 + "=? and " + field_1 + "=?");
			
			// Establecemos el valor del atributo.
			query.setParameter(0, value_0, Hibernate.INTEGER);
			query.setParameter(1, value_1, Hibernate.STRING);
			
			// Ejecutamos la consulta.
			List results = query.list();
			
			// Comprobamos el n&uacute;mero de resultados.
			if (results.size() == 1)
			{
				// Si el n&uacute;mero de elementos es 1 existe el valor.
				return results.get(0);
			}
		}
		
		// No existe valor.
		return null;
	}
	
	/**
	 * <p>Este m&eacute;todo realiza una consulta sobre la tabla <i>table</i> y obtiene el registro
	 * que contiene los campos <i>field_0, field_1</i>  con los valores de los par&aacute;metros <i>value_0, value_1</i> respectivamente.
	 * En caso de no existir o haber m&aacute;s de uno retorna nulo</p>
	 *  
	 * @param session <code>Session</code> sesi&oacute;n de Hibernate.
	 * @param table <code>Table</code> la tabla de la base de datos.
	 * @param field_0, field_1 <code>String</code> campos a comprobar.
	 * @param value_0, value_1 <code>int</code> valor a buscar.
	 * @return <code>Object</code>  contiene la informacion del objeto dentro de 
	 * la base de datos o <code>null</code> en caso de no existir.
	 */
	@SuppressWarnings("rawtypes")
	protected Object existRegister(Session session, Table table, String field_0, int value_0, String field_1, int value_1)
	{
		
		// Comprobamos que los par&aacute;metros no son null y adem&aacute;s que se ha inicializado la sesi&oacute;n
		if ((session.isOpen()) && (table != null) && (!Validations.isEmpty(field_0)) && (!Validations.isEmpty(field_1)))
		{
			
			// Preparamos la consulta HQL.
			Query query = session.createQuery("from " + table + " where " + field_0 + "=? and " + field_1 + "=?");
			
			// Establecemos el valor del atributo.
			query.setParameter(0, value_0, Hibernate.INTEGER);
			query.setParameter(1, value_1, Hibernate.INTEGER);
			
			// Ejecutamos la consulta.
			List results = query.list();
			
			// Comprobamos el n&uacute;mero de resultados.
			if (results.size() == 1)
			{
				// Si el n&uacute;mero de elementos es 1 existe el valor.
				return results.get(0);
			}
		}
		
		// No existe valor.
		return null;
	}
	
	/**
	 * <p>Este m&eacute;todo realiza una consulta sobre la tabla <i>table</i> y obtiene el registro
	 * que contiene los campos <i>field_0, field_1,field_2</i>  con los valores de los par&aacute;metros <i>value_0, value_1,value_2</i> respectivamente.
	 * En caso de no existir o haber m&aacute;s de uno retorna nulo</p>
	 *  
	 * @param session <code>Session</code> sesi&oacute;n de Hibernate.
	 * @param table <code>Table</code> la tabla de la base de datos.
	 * @param field_0, field_1, field_2 <code>String</code> campos a comprobar.
	 * @param value_0, value_1, value_2 <code>int</code> valor a buscar.
	 * @return <code>Object</code>  contiene la informacion del objeto dentro de 
	 * la base de datos o <code>null</code> en caso de no existir.
	 */
	@SuppressWarnings("rawtypes")
	protected Object existRegister(Session session, Table table, String field_0, int value_0, String field_1, int value_1, String field_2, int value_2)
	{
		
		// Comprobamos que los par&aacute;metros no son null y adem&aacute;s que se ha inicializado la sesi&oacute;n
		if ((session.isOpen()) && (table != null) 
				&& (!Validations.isEmpty(field_0)) && (!Validations.isEmpty(field_1)) && (!Validations.isEmpty(field_2)))
		{
			
			// Preparamos la consulta HQL.
			Query query = session.createQuery("from " + table + " where " + field_0 + "=? and " + field_1 + "=? and " + field_2 + "=?");
			
			// Establecemos el valor del atributo.
			query.setParameter(0, value_0, Hibernate.INTEGER);
			query.setParameter(1, value_1, Hibernate.INTEGER);
			query.setParameter(2, value_2, Hibernate.INTEGER);
			
			// Ejecutamos la consulta.
			List results = query.list();
			
			// Comprobamos el n&uacute;mero de resultados.
			if (results.size() == 1)
			{
				// Si el n&uacute;mero de elementos es 1 existe el valor.
				return results.get(0);
			}
		}
		
		// No existe valor.
		return null;
	}
		
	/**
	 * <p>Este m&eacute;todo realiza una consulta sobre la tabla <i>table</i> y obtiene el registro
	 * que contiene el campo <i>field_0</i>  con el valor del par&aacute;metro <i>value_0</i>.
	 * En caso de no existir o haber m&aacute;s de uno retorna nulo</p>
	 *  
	 * @param session <code>Session</code> sesi&oacute;n de Hibernate.
	 * @param table <code>Table</code> la tabla de la base de datos.
	 * @param field_0 <code>String</code> campo a comprobar.
	 * @param value_0 <code>String</code> valor a buscar.
	 * @return <code>Object</code>  contiene la informacion del objeto dentro de 
	 * la base de datos o <code>null</code> en caso de no existir.
	 */
	@SuppressWarnings("rawtypes")
	protected Object existRegister(Session session, Table table, String field_0, String value_0)
	{
		
		// Comprobamos que los par&aacute;metros no son null y adem&aacute;s que se ha inicializado la sesi&oacute;n
		if ((session.isOpen()) && (table != null) && (!Validations.isEmpty(field_0)))
		{
			// Preparamos la consulta HQL.
			Query query = session.createQuery("from " + table + " where " + field_0 + "=?");
			
			// Establecemos el valor del atributo.
			query.setParameter(0, value_0, Hibernate.STRING);
			
			// Ejecutamos la consulta.
			List results = query.list();
			
			// Comprobamos el n&uacute;mero de resultados.
			if (results.size() == 1)
			{
				// Si el n&uacute;mero de elementos es 1 existe el valor.
				return results.get(0);
			}
		}
		
		// No existe valor.
		return null;
	}

	/**
	 * <p>Este m&eacute;todo realiza una consulta sobre la tabla <i>table</i> y obtiene el registro
	 * que contiene el campo <i>field_0</i>  con el valor del par&aacute;metro <i>value_0</i>.
	 * En caso de no existir o haber m&aacute;s de uno retorna nulo</p>
	 *  
	 * @param session <code>Session</code> sesi&oacute;n de Hibernate.
	 * @param table <code>Table</code> la tabla de la base de datos.
	 * @param field_0 <code>String</code> campo a comprobar.
	 * @param value_0 <code>int</code> valor a buscar.
	 * @return <code>Object</code>  contiene la informacion del objeto dentro de 
	 * la base de datos o <code>null</code> en caso de no existir.
	 */
	@SuppressWarnings("rawtypes")
	protected Object existRegister(Session session, Table table, String field_0, int value_0)
	{
		
		// Comprobamos que los par&aacute;metros no son null y adem&aacute;s que se ha inicializado la sesi&oacute;n
		if ((session.isOpen()) && (table != null) && (!Validations.isEmpty(field_0)))
		{
			// Preparamos la consulta HQL.
			Query query = session.createQuery("from " + table + " where " + field_0 + "=?");
			
			// Establecemos el valor del atributo.
			query.setParameter(0, value_0, Hibernate.INTEGER);
			
			// Ejecutamos la consulta.
			List results = query.list();
			
			// Comprobamos el n&uacute;mero de resultados.
			if (results.size() == 1)
			{
				// Si el n&uacute;mero de elementos es 1 existe el valor.
				return results.get(0);
			}
		}
		
		// No existe valor.
		return null;
	}
	
	/**
	 * <p>Retorna todos los objetos de la tabla <i>table</i>. Si hay alg&uacute;n error
	 * retornar&aacute; nulos</p>
	 * 
	 * @param <code>Session</code> sesi&oacute;n de Hibernate.
	 * @param table <code>Table</code>  la tabla de la base de datos.
	 * @return <code>List<Object></code> con todos los objetos de la tabla.
	 */
	@SuppressWarnings("unchecked")
	protected List<Object> getRegisters(Session session, Table table)
	{
		// Verificamos par&aacute;metros de llamada
		if ((session != null) && (session.isOpen()) && (table != null))
		{
			//Preparamos sentencia Query.
			Query query = session.createQuery("from " + table);
			
			//Ejecutamos Query.
			return query.list();
		}
		
		//Hay errores en los par&aacute;metros recibidos, retornamos una lista vac&iacute;a 
		return (new ArrayList<Object>());
	}
	
	/**
	 * <p>Retorna todos los objetos de la tabla <i>table</i>, donde el campo <i>field_0<i> toma el valor
	 * <i>value_0<i>. Si hay alg&uacute;n error retornar&aacute; nulos</p>
	 * 
	 * @param <code>Session</code> sesi&oacute;n de Hibernate.
	 * @param table <code>Table</code>  la tabla de la base de datos.
	 * @return <code>List<Object></code> con todos los objetos de la tabla.
	 */
	@SuppressWarnings("unchecked")
	protected List<Object> getRegisters(Session session, Table table, String field_0, int value_0)
	{
		// Verificamos par&aacute;metros de llamada
		if ((session != null) && (session.isOpen()) && (table != null) && (!Validations.isEmpty(field_0)))
		{
			
			//Preparamos sentencia Query.
			Query query = session.createQuery("from " + table  + " where " + field_0 + "=?");
			
			// Establecemos el valor del atributo.
			query.setParameter(0, value_0, Hibernate.INTEGER);
			
			//Ejecutamos Query.
			return query.list();
		}
		
		//Hay errores en los par&aacute;metros recibidos, retornamos una lista vac&iacute;a 
		return (new ArrayList<Object>());
	}
	
	/**
	 * <p>Cierra la sesi&oacute;n de Hibernate que se le pasa como par&aacute;metro.</p> 
	 * 
	 * @param session <code>Session</code> la sesi&oacute;n que se quiere cerrar.
	 * @throws ExceptionControl excepciones
	 */
	protected void closeSession(Session session) throws ExceptionControl
	{
		// Comprobamos que el par&aacute;metro no es nulo y adem&aacute;s que la sesi&oacute;n est&aacute; abierta.
		if ((session != null) && (session.isOpen()))
		{
			try
			{
				// Intentamos cerrar la sesi&oacute;n
				session.close();
			}
			catch (Exception ex)
			{
				// Propagamos la excepci&oacute;n.
				throw (new ExceptionControl(ex.getMessage()));
			}
		}
	}
	
}
