package edu.uoc.integra.business.functions;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Date;
import java.util.List;
import java.util.Set;

import org.hibernate.Session;

import edu.uoc.integra.business.exceptions.ExceptionControl;
import edu.uoc.integra.business.interfaces.IBProjectManager;
import edu.uoc.integra.business.objects.BOProject;
import edu.uoc.integra.business.objects.BORol;
import edu.uoc.integra.business.objects.BOState;
import edu.uoc.integra.business.objects.BOUser;
import edu.uoc.integra.business.objects.BOUserRol;
import edu.uoc.integra.business.objects.BOWorkFlow;
import edu.uoc.integra.business.objects.BOWorkFlow_State;
import edu.uoc.integra.business.objects.transfers.TOProject;
import edu.uoc.integra.business.objects.transfers.TORol;
import edu.uoc.integra.business.objects.transfers.TOState;
import edu.uoc.integra.business.objects.transfers.TOUserRol;
import edu.uoc.integra.business.objects.transfers.TOWorkFlow;

/**
 * <p>Implementa las funcionalidades ofrecidas por la interfaz <code>IBProjectManager</code> </p>
 * 
 * @author Miguel Angel Pineda Cruz
 * @version 1.0 
 * @see IBProjectManager
 */
public class IBProjectManagerImpl extends IBCommonManagerImpl implements IBProjectManager 
{
	
	//************************************************************//
	// CONSTRUCTOR.                                               //
	//************************************************************//
	/**
	 * <p><code>Construye un nuevo objeto <code>IBProjectManagerImpl</code> e inicializa el 
	 * marco de persistencia <i>Hibernate</i>.</p>
	 * @throws ExceptionControl error al iniciar el marco de persistencia.
	 */
	public IBProjectManagerImpl() throws ExceptionControl
	{
		// Constructor de la superclase.
		super();
	}
	
	/**
	 * <p>Este M&eacute;todo es llamadado cada vez que se quiera dar de alta a un proyecto
	 * a partir de la informaci&oacute;n recibida por par&aacute;metro.</p>
	 * 
	 * @param project <code>TOProject</code> Informaci&oacute;n del Proyecto a crear.
	 * 
	 * @return <code>TOProject</code> contiene la informaci&oacute;n del proyecto creado</p>
	 * 
	 */
	@Override
	public TOProject createProject(TOProject project) throws ExceptionControl {
		
		// Par&aacute;metro nulo
		if (project == null) throw (new ExceptionControl(ExceptionControl.MSG_PROJECT_PARAM_NULL));
		
		// Abrimos la sesi&oacute;n de Hibernate.
		Session session = this.sessionFactory.openSession();
		
		try
		{
			System.out.println("Verificamos si prefix_code del proyecto registrado");
			// Comprobamos si "prefix_code" ya se encuentra registrado.
			if (existRegister(session, Table.PROJECT, "prefix_code", project.getPrefix_code()) != null)
			{
				// Lanzamos Excepci&oacute;n
				throw (new ExceptionControl(ExceptionControl.MSG_PROJECT_PREFIX_CODE_REGISTER));
			}
			
			System.out.println("Verificamos si name del proyecto registrado");
			// Comprobamos si "name" ya se encuentra registrado.
			if (existRegister(session, Table.PROJECT, "name", project.getName()) != null)
			{
				// Lanzamos Excepci&oacute;n
				throw (new ExceptionControl(ExceptionControl.MSG_PROJECT_NAME_REGISTER));
			}
			
			System.out.println("Instanciamos clase BOProject");
			// Iniciamos proceso de creaci&oacute;n
			// Creamos un objeto BOProject a partir del valor del par&aacute;metro.
			BOProject boProject = new BOProject(project);
			
			System.out.println("Actualizamos");
			// Grabamos la informaci&oacute;n en B.D.
			session.saveOrUpdate(boProject);
			
			System.out.println("Realizamos Commit");
			// Realizamos commit.
			session.flush();
			
			System.out.println("Retornamos Objeto Actualizado");
			// Retornamos la informaci&oacute;n actualizada
			return boProject.getTOProject();
		}
		finally
		{
			System.out.println("Cerramos Sesion");
			// Cerramos la sesi&oacute;n de Hibernate.
			closeSession(session);
		}
	}

		
	/**
	 * <p>Este m&eacute;todo es llamado cada vez que se quiere consultar la lista de
	 * proyectos registrados en el sistema.</p>
	 * 
	 * @return <code>List<TOProject></code> lista de proyectos
	 * 
	 */
	@Override
	public List<TOProject> listProject() throws ExceptionControl {

		
		System.out.println("Abrimos sesi&oacute;n de Hibernate");
		
		// Abrimos una sesi&oacute;n de Hibernate.
		Session session = sessionFactory.openSession();
		
		try
		{
			System.out.println("Recuperamos todos los registros");
			
			// Obtenemos la lista de todos los proyectos registrados.
			List<Object> list = getRegisters(session, Table.PROJECT);
			
			System.out.println("Construimos lista");
			
			// Construimos la lista con los objetos que ser&aacute;n retornados.
			List<TOProject> projects = new ArrayList<TOProject>();
			
			// Recorremos todos los proyectos.
			for (Object objBOProject : list)
			{
				// Comprobamos que el objeto es de tipo BOProject.
				if (objBOProject instanceof BOProject)
				{
					// Realizamos el cast del objeto
					BOProject boProject = (BOProject) objBOProject;
					
					// A&ntilde;adimos a la lista el TOProject equivalente a BOProject.
					projects.add(boProject.getTOProject());
				}
			}
			
			System.out.println("Ordenamos lista");
			// Una vez tenemos todos los proyectos los ordenamos por "id"
			Collections.sort(projects);
			
			// Retornamos la lista de proyectos.
			return projects;
		}
		catch (Exception e)
		{
			
			System.out.println("Se ha producido un error:" + e.getMessage());
			// Lanzamos Excepci&oacute;n
			throw new ExceptionControl(e.getMessage());
		}
		finally
		{
			System.out.println("Cerramos lista");
			
			// Cerramos la sesi&oacute;n de Hibernate
			closeSession(session);
		}
	}

	
	/**
	 * <p>M&eacute;todo para listar los proyectos registrados en el sistema que cumplan los criterios que se han
	 * pasado por par&aacute;metro</p>
	 * 
	 * @param prefix_code <code>String</code> proyecto que contiene este c&oacute;digo
	 * @param name <code>String</code> proyecto que contiene este nombre
	 * 
	 * @return List<TOProject> contiene la informaci&oacute;n de los proyectos que cumplen los criterios
	 * 
	 * @throws ExceptionControl excepci&oacute;n generada por persistencia
	 *  
	 */
	@Override
	public List<TOProject> listProject(String prefix_code, String name) throws ExceptionControl {

		// Abrimos una sesi&oacute;n de Hibernate.
		Session session = sessionFactory.openSession();
		
		try
		{
			// Obtenemos la lista de todos los proyectos registrados.
			List<Object> list = getRegisters(session, Table.PROJECT);
			
			// Construimos la lista con los objetos que ser&aacute;n retornados.
			List<TOProject> projects = new ArrayList<TOProject>();
			
			// Recorremos todos los proyectos.
			for (Object objBOProject : list)
			{
				// Comprobamos que el objeto es de tipo BOProject.
				if (objBOProject instanceof BOProject)
				{
					// Realizamos el cast del objeto
					BOProject boProject = (BOProject) objBOProject;
					
					// Comprobamos si el objeto recuperado cumple con
					// las condiciones prefijadas por par&aacute;metro
					if(boProject.getPrefix_code().contains(prefix_code) &&
					   boProject.getName().contains(name))
					  // A&ntilde;adimos a la lista el TOProject equivalente a BOProject.
						projects.add(boProject.getTOProject());
				}
			}
			
			// Una vez tenemos todos los proyectos los ordenamos por "id"
			Collections.sort(projects);
			
			// Retornamos la lista de proyectos filtrados.
			return projects;
		}
		catch (Exception e)
		{
			// Lanzamos Excepci&oacute;n
			throw new ExceptionControl(e.getMessage());
		}
		finally
		{
			// Cerramos la sesi&oacute;n de Hibernate
			closeSession(session);
		}
	}
	
	/**
	 * <p>Este M&eacute;todo es llamado cada vez que se quiere modificar  la informaci&oacute;n de
	 * un proyecto a partir de la informaci&oacute;n recibida por par&aacute;metro </p>
	 * 
	 * @param idProject <code>int</code> identificador del proyecto.
	 * @param prefix_code <code>String</code> codigo del proyecto.
	 * @param name <code>String</code> nombre  del proyecto.
	 * @param descript <code>String</code> descripci&oacute;n del proyecto.
	 * @param fecopen <code>Date</code> fecha inicio del proyecto.
	 * @param fecclose <code>Date</code> fecha fin del proyecto.
	 * 
	 * @return <code>TOProject</code> informaci&oacute;n del proyecto actualizada
	 *  
	 */
	@Override
	public TOProject modifyProject(int idProject, String prefix_code, String name, String descript, Date fecopen, Date fecclose)
			throws ExceptionControl {
		
			// Abrimos la sesi&oacute;n de Hibernate.
			Session session = this.sessionFactory.openSession();
			
			try
			{
				// Comprobamos que el proyecto est&aacute; registrado 
				   Object objBOProject = existRegister(session, Table.PROJECT, "id", idProject);
				   
				// Comprobamos que no es nulo y adem&aacute;s es de tipo BOProject
				   if ((objBOProject != null) && (objBOProject instanceof BOProject))
					{		
						// Realizamos el cast del objeto
						BOProject boProject = (BOProject) objBOProject;
												
						//Comprobamos que no existe el prefix_code asignado a otro proyecto
						if(!boProject.getPrefix_code().equals(prefix_code) && 
								existRegister(session, Table.PROJECT, "prefix_code", prefix_code) != null){
							//El prefix_code se encuentra registrado con otro proyecto
							throw (new ExceptionControl(ExceptionControl.MSG_PROJECT_PREFIX_CODE_REGISTER));
						}
						
						//Comprobamos que no existe el name asignado a otro proyecto
						if(!boProject.getName().equals(name) && 
								existRegister(session, Table.PROJECT, "name", name) != null){
							//El prefix_code se encuentra registrado con otro proyecto
							throw (new ExceptionControl(ExceptionControl.MSG_PROJECT_NAME_REGISTER));
						}
																		
						// Actualizamos la Informaci&oacute;n del Proyecto
						boProject.setPrefix_code(prefix_code.trim());
						boProject.setName(name.trim());
						boProject.setDescript(descript.trim());
						boProject.setFecopen(fecopen);
						boProject.setFecclose(fecclose);
						
						// Actualizamos la informaci&oacute;n del proyecto.
						session.saveOrUpdate(boProject);
						// Commit
						session.flush();
						// Retornamos un objeto TOProject con la informaci&oacute;n actualizada.
						return boProject.getTOProject();
					}
				   
				// Identificador no V&aacute;lido
					throw (new ExceptionControl(ExceptionControl.MSG_PROJECT_ID_INVALID));
			}
			catch (Exception ex)
			{
				// Propagamos la excepci&oacute;n
				throw (new ExceptionControl(ex.getMessage()));
			}
			finally
			{
			 // Cerramos la sesi&oacute;n de Hibernate.
				closeSession(session);
			}
	}

	
	/**
	 * <p>Este M&eacute;todo es llamado cada vez que se quiere dar de baja a un proyecto
	 * del sistema a partir de la informaci&oacute;n recibida por par&aacute;metro</p>
	 * 
	 * @param idProject <code>int</code> identificador del proyecto.
	 * 
	 */
	@Override
	public void deleteProject(int idProject) throws ExceptionControl {
		
		// Abrimos la sesi&oacute;n de Hibernate.
		   Session session = sessionFactory.openSession();
		   
		   try
			{
			// Comprobamos que el proyecto est&aacute; registrado 
			   Object objBOProject = existRegister(session, Table.PROJECT, "id", idProject);
			   
			// Comprobamos que no es nulo y adem&aacute;s es de tipo BOProject.
			   if ((objBOProject != null) && (objBOProject instanceof BOProject))
			   {
				 // Realizamos el cast del objeto
				 BOProject boProject = (BOProject) objBOProject;
				 
				 // Borramos los Roles del Proyecto
				 System.out.println("Obtenemos todos los roles asignados al proyecto");
				   
				 // Obtenemos la lista de todos los roles que tienen asignado este proyecto
				 List<Object> list = getRegisters(session, Table.ROL, "idproject", boProject.getId());
				   
				 System.out.println("Roles en este proyecto:" +  list.size());
				   
				 // Recorremos todos los roles recuperados.
				 for (Object objBORol : list)
				 {
				   // Comprobamos que el objeto es de tipo BORol.
				   if (objBORol instanceof BORol)
				   {
					   // Realizamos el cast del objeto
					   BORol boRol = (BORol) objBORol;
					   
					   session.delete(boRol);
				   }
				 }
				 
				 // Commit.
				 session.flush();
				 
				 // Borramos Estados del Proyecto
				 System.out.println("Obtenemos todos los estados asignados al proyecto");
				   
				 // Obtenemos la lista de todos los roles que tienen asignado este proyecto
				 list = getRegisters(session, Table.STATE, "idproject", boProject.getId());
				   
				 System.out.println("Estados en este proyecto:" +  list.size());
				   
				 // Recorremos todos los estados recuperados.
				 for (Object objBOState : list)
				 {
				   // Comprobamos que el objeto es de tipo BOState.
				   if (objBOState instanceof BOState)
				   {
					   // Realizamos el cast del objeto
					   BOState boState = (BOState) objBOState;
					   
					   session.delete(boState);
				   }
				 }
				 
				 // Commit.
				 session.flush();
				 
				 // Borramos Workflows del Proyecto
				 System.out.println("Obtenemos todos los workflows asignados al proyecto");
				   
				 // Obtenemos la lista de todos los roles que tienen asignado este proyecto
				 list = getRegisters(session, Table.WORKFLOW, "idproject", boProject.getId());
				   
				 System.out.println("WorkFlows en este proyecto:" +  list.size());
				   
				 // Recorremos todos los estados recuperados.
				 for (Object objBOWorkFlow : list)
				 {
				   // Comprobamos que el objeto es de tipo BOWorkFlow.
				   if (objBOWorkFlow instanceof BOWorkFlow)
				   {
					   // Realizamos el cast del objeto
					   BOWorkFlow boWorkFlow = (BOWorkFlow) objBOWorkFlow;
					   
					   
					 //Recuperamos  los estados asignados al WorkFlow
					   Set<BOWorkFlow_State> boWorkFlow_States = boWorkFlow.getIdstate_next();
								   
					 //Recorremos el conjunto de estados pr&oacute;ximos del WorkSpace
					   for (BOWorkFlow_State boWorkFlow_State : boWorkFlow_States)
					   {
						  session.delete(boWorkFlow_State);
										   
						}
								   
					 //Commit.
					   session.flush();
					   
					   session.delete(boWorkFlow);
				   }
				 }
				 
				 // Commit.
				 session.flush();
				 
				 
				 // Borramos Usuarios-Roles del Proyecto
				 System.out.println("Obtenemos los usuarios-roles asignados al proyecto");
				   
				 // Obtenemos la lista de todos los usuarios que tienen asignado este rol
				 list = getRegisters(session, Table.USERROL, "idproject", boProject.getId());
				   
				 System.out.println("Usuarios-Roles en este proyecto:" +  list.size());
				   
				 // Recorremos todos los usuarios-roles recuperados.
				 for (Object objBOUserRol : list)
				 {
				   // Comprobamos que el objeto es de tipo BOUserRol.
				   if (objBOUserRol instanceof BOUserRol)
				   {
					   // Realizamos el cast del objeto
					   BOUserRol boUserRol = (BOUserRol) objBOUserRol;
						   
					   session.delete(boUserRol);
				   }
				 }
				 
				 // Commit.
				 session.flush();
				 
				// Borramos proyecto
				 session.delete(boProject);
				 
				// Commit.
				 session.flush();
					
				}
				else 
				{
				 // Identificador no V&aacute;lido
					throw (new ExceptionControl(ExceptionControl.MSG_PROJECT_ID_INVALID));
				}
			}		
		   catch (Exception ex)
			{
				// Propagamos la excepci&oacute;n
				throw (new ExceptionControl(ex.getMessage()));
			}
			finally
			{
			 // Cerramos la sesi&oacute;n de Hibernate.
				closeSession(session);
			}
		
	}

	/**
	 * <p>Este M&eacute;todo es llamado cada vez que se quiere consultar  la informaci&oacute;n de
	 * un proyecto a partir de la informaci&oacute;n recibida por par&aacute;metro </p>
	 * 
	 * @param idProject <code>int</code> identificador del proyecto.
	 * 
	 * @return <code>TOProject</code> contiene la informaci&oacute;n del proyecto cuyo
	 * <i>id</i> es igual al que se ha recibido como par&aacute;metro.
	 * 
	 */
	@Override
	public TOProject getProject(int idProject) throws ExceptionControl {
		

		   System.out.println("Abrimos sesi&oacute;n de Hibernate");

		   // Abrimos la sesi&oacute;n de Hibernate.
			Session session = this.sessionFactory.openSession();
			
			try
			{
				System.out.println("Comprobamos que el proyecto est&aacute; registrado");
				// Comprobamos que el proyecto est&aacute; registrado
				Object objBOProject = existRegister(session, Table.PROJECT, "id", idProject);
				
				// Comprobamos que no es nulo y adem&aacute;s es de tipo BOProject.
				if ((objBOProject != null) && (objBOProject instanceof BOProject))
				{
					System.out.println("Casteamos a objBOProject");
					 // Realizamos el cast del objeto
					 BOProject boProject = (BOProject) objBOProject;
					
					 System.out.println("Retornamos Datos de Proyecto");
					 //Retornamos la informaci&oacute;n del objeto en un objeto TOProject
					return boProject.getTOProject();
					
				}
				
				System.out.println("Proyecto no existe");
				 // Identificador no V&aacute;lido
				throw (new ExceptionControl(ExceptionControl.MSG_PROJECT_ID_INVALID));
			}
			finally
			{
				System.out.println("Cerramos sesi&oacute;n");
				// Cerramos la sesi&oacute;n de Hibernate.
				closeSession(session);
			}
	}

	/**
	 * <p>Este M&eacute;todo es llamadado cada vez que se quiera dar de alta a un rol dentro de un proyecto
	 * a partir de la informaci&oacute;n recibida por par&aacute;metro.</p>
	 * 
	 * @param rol <code>TORol</code> Informaci&oacute;n del Rol de un Projecto.
	 * 
	 * @return <code>TORol</code> contiene la informaci&oacute;n del rol creado</p>
	 * 
	 */
	public TORol createRolProject(TORol rol)
			throws ExceptionControl {
		
		System.out.println("Entramos en M&eacute;todo createRolProject");
		// Par&aacute;metro nulo
		if (rol == null) throw (new ExceptionControl(ExceptionControl.MSG_PROJECT_PARAM_NULL));
		
		System.out.println("Abrimos sesi&oacute;n de Hibernate");
		// Abrimos la sesi&oacute;n de Hibernate.
		Session session = this.sessionFactory.openSession();
		
		try
		{
			
			System.out.println("Verificamos si el proyecto est&aacute; registrado");
			// Comprobamos que el proyecto est&aacute; registrado
			Object objBOProject = existRegister(session, Table.PROJECT, "id", rol.getIdproject());
			
			if (objBOProject== null || !(objBOProject instanceof BOProject))
			{
				// Lanzamos Excepci&oacute;n
				throw (new ExceptionControl(ExceptionControl.MSG_PROJECT_ID_INVALID));
				
			}
			
			System.out.println("Verificamos si name del rol registrado para el proyecto");
			// Comprobamos si "name" ya se encuentra registrado para el id del proyecto.
			if (existRegister(session, Table.ROL, "idproject", rol.getIdproject(), "name", rol.getName()) != null)
			{
				// Lanzamos Excepci&oacute;n
				throw (new ExceptionControl(ExceptionControl.MSG_ROL_NAME_REGISTER));
			}
						
			System.out.println("Casteamos a objBOProject");
			// Realizamos el cast del objeto
			BOProject boProject = (BOProject) objBOProject;
			
			System.out.println("Instanciamos clase BORol");
			// Iniciamos proceso de creaci&oacute;n
			// Creamos un objeto BORol a partir del valor del par&aacute;metro.
			BORol boRol = new BORol(boProject,rol.getName());
			
			System.out.println("Actualizamos");
			// Grabamos la informaci&oacute;n en B.D.
			session.saveOrUpdate(boRol);
			
			System.out.println("Realizamos Commit");
			// Realizamos commit.
			session.flush();
			
			System.out.println("Retornamos Objeto Actualizado");
			// Retornamos la informaci&oacute;n actualizada
			return boRol.getTORol();
		}
		finally
		{
			System.out.println("Cerramos Sesion");
			// Cerramos la sesi&oacute;n de Hibernate.
			closeSession(session);
		}
	}

	/**
	 * <p>M&eacute;todo para listar los roles de un proyecto registrado en el sistema</p>
	 * 
	 * @param project <code>TOProject</code> informaci&oacute;n del proyecto que queremos consultar sus roles
	 * 
	 * @return List<TORol> contiene la informaci&oacute;n de los estados definidos en el proyecto
	 * 
	 * @throws ExceptionControl excepci&oacute;n generada por persistencia
	 *  
	 */
	@Override
	public List<TORol> listRolProject(TOProject project)
			throws ExceptionControl {
		
		// Abrimos una sesi&oacute;n de Hibernate.
		Session session = sessionFactory.openSession();
		
		try
		{
			System.out.println("Verificamos si el proyecto est&aacute; registrado");
			// Comprobamos que el proyecto est&aacute; registrado
			Object objBOProject = existRegister(session, Table.PROJECT, "id", project.getId());
			
			if (objBOProject== null || !(objBOProject instanceof BOProject))
			{
				// Lanzamos Excepci&oacute;n
				throw (new ExceptionControl(ExceptionControl.MSG_PROJECT_ID_INVALID));
				
			}
			
			System.out.println("Obtenemos los roles registrados en un proyecto");
			// Obtenemos la lista de todos los proyectos registrados.
			List<Object> list = getRegisters(session, Table.ROL, "idproject", project.getId());
			
			System.out.println("Construimos la lista de los roles de un proyecto");
			// Construimos la lista con los objetos que ser&aacute;n retornados.
			List<TORol> rols = new ArrayList<TORol>();
			
			// Recorremos todos los roles recuperados.
			for (Object objBORol : list)
			{
				// Comprobamos que el objeto es de tipo BORol.
				if (objBORol instanceof BORol)
				{
					// Realizamos el cast del objeto
					BORol boRol = (BORol) objBORol;
					// A&ntilde;adimos a la lista el TORol equivalente a BORol.
					rols.add(boRol.getTORol());
				}
			}
			
			System.out.println("Ordenamos la lista de los roles de un proyecto");
			// Una vez tenemos todos los roles los ordenamos por "name"
			Collections.sort(rols);
			
			System.out.println("retornamos la lista de los roles de un proyecto");
			// Retornamos la lista de roles ordenados.
			return rols;
		}
		catch (Exception e)
		{
			// Lanzamos Excepci&oacute;n
			throw new ExceptionControl(e.getMessage());
		}
		finally
		{
			// Cerramos la sesi&oacute;n de Hibernate
			closeSession(session);
		}
	}
	
	/**
	 * <p>Este M&eacute;todo es llamado cada vez que se quiere modificar  la informaci&oacute;n de
	 * un rol de un proyecto a partir de la informaci&oacute;n recibida por par&aacute;metro </p>
	 * 
	 * @param idProject <code>int</code> identificador del proyecto.
	 * @param idRol <code>int</code> identificador del rol.
	 * @param nameRol <code>String</code> nombre del rol
	 * 
	 * @return <code>TORol</code> informaci&oacute;n del rol actualizada
	 *  
	 */
	@Override
	public TORol modifyRolProject(int idProject, int idRol, String nameRol) throws ExceptionControl {
		
		// Abrimos la sesi&oacute;n de Hibernate.
		Session session = this.sessionFactory.openSession();
					
		try
		{
			// Comprobamos que el proyecto est&aacute; registrado 
			   Object objBOProject = existRegister(session, Table.PROJECT, "id", idProject);
			   
			// Comprobamos que el rol est&aacute; registrado 
			   Object objBORol = existRegister(session, Table.ROL, "id", idRol);
			   		   
			// Comprobamos que no es nulo y adem&aacute;s es de tipo BOProject.
			// Comprobamos que no es nulo y adem&aacute;s es de tipo BORol.
			   if ((objBOProject != null) && (objBOProject instanceof BOProject) &&
				   (objBORol != null) && (objBORol instanceof BORol))
					{
						
				   System.out.println("Casteamos a objBOProject");
				   // Realizamos el cast del objeto
				   BOProject boProject = (BOProject) objBOProject;
							
				   System.out.println("Casteamos a objBORol");
				   // Realizamos el cast del objeto
				   BORol boRol = (BORol) objBORol;
					
				   //Verificamos que el Rol solicitado es del proyecto que hemos
				   //recibido por par&aacute;metro
				   if(boRol.getIdproject().getId() != boProject.getId()){
						
						System.out.println("Identificador de Proyecto no coincide con el de rol");
						// Identificador no V&aacute;lido
						throw (new ExceptionControl(ExceptionControl.MSG_PROJECT_ID_INVALID));
						
				   }
				   
				   //Comprobamos que no existe el name asignado a otro rol en el proyecto
				   if(!boRol.getName().equals(nameRol) && 
					   existRegister(session, Table.ROL, "name", nameRol) != null){
						//El name se encuentra registrado con otro rol en el proyecto
						throw (new ExceptionControl(ExceptionControl.MSG_ROL_NAME_REGISTER));
					}
																				
					// Actualizamos la Informaci&oacute;n del Rol
					boRol.setName(nameRol.trim());
								
					// Actualizamos la informaci&oacute;n del proyecto.
					session.saveOrUpdate(boRol);
					
					// Commit
					session.flush();
					
					// Retornamos un objeto TORol con la informaci&oacute;n actualizada.
					return boRol.getTORol();
				}
						   
				// Identificador no V&aacute;lido
				   throw (new ExceptionControl(ExceptionControl.MSG_PROJECT_ID_INVALID));
			}
			catch (Exception ex)
			{
				// Propagamos la excepci&oacute;n
				throw (new ExceptionControl(ex.getMessage()));
			}
			finally
			{
			 // Cerramos la sesi&oacute;n de Hibernate.
				closeSession(session);
			}
	}
	
	/**
	 * <p>Este M&eacute;todo es llamado cada vez que se quiere dar de baja a un rol de un proyecto
	 * del sistema a partir de la informaci&oacute;n recibida por par&aacute;metro</p>
	 * 
	 * @param idProject <code>int</code> identificador del proyecto.
	 * @param idRol     <code>int</code> identificador del rol del proyecto. 
	 * 
	 */
	@Override
	public void deleteRolProject(int idProject, int idRol)
			throws ExceptionControl {
		
		// Abrimos la sesi&oacute;n de Hibernate.
		Session session = sessionFactory.openSession();
		   
		try
		{
			// Comprobamos que el proyecto est&aacute; registrado 
			   Object objBOProject = existRegister(session, Table.PROJECT, "id", idProject);
			   
			// Comprobamos que el rol est&aacute; registrado 
			   Object objBORol = existRegister(session, Table.ROL, "id", idRol);
			
			// Comprobamos que no es nulo y adem&aacute;s es de tipo BOProject.
			// Comprobamos que no es nulo y adem&aacute;s es de tipo BORol.
			   if ((objBOProject != null) && (objBOProject instanceof BOProject) &&
				   (objBORol != null) && (objBORol instanceof BORol))
					{
					
				   System.out.println("Casteamos a objBOProject");
				   // Realizamos el cast del objeto
				   BOProject boProject = (BOProject) objBOProject;
						
				   System.out.println("Casteamos a objBORol");
				   // Realizamos el cast del objeto
				   BORol boRol = (BORol) objBORol;
				   
				   //Verificamos que el Rol solicitado es del proyecto que hemos
				   //recibido por par&aacute;metro
				   if(boRol.getIdproject().getId() != boProject.getId()){
						
						System.out.println("Identificador de Proyecto no coincide con el de rol");
						// Identificador no V&aacute;lido
						throw (new ExceptionControl(ExceptionControl.MSG_PROJECT_ID_INVALID));
						
				   }else{

					   System.out.println("Obtenemos los usuarios que tienen este rol asignado");
					   
					   // Obtenemos la lista de todos los usuarios que tienen asignado este rol
					   List<Object> list = getRegisters(session, Table.USERROL, "idrol", boRol.getId());
					   
					   System.out.println("Usuarios con este rol:" +  list.size());
					   
					   // Recorremos todos los usuarios-roles recuperados.
					   for (Object objBOUserRol : list)
					   {
						   // Comprobamos que el objeto es de tipo BOUserRol.
						   if (objBOUserRol instanceof BOUserRol)
						   {
							   // Realizamos el cast del objeto
							   BOUserRol boUserRol = (BOUserRol) objBOUserRol;
							   
							   session.delete(boUserRol);
						   }
					   }
					   
					   System.out.println("Obtenemos los workspace que tienen este rol asignado");
					   
					   // Obtenemos la lista de todos los workspace que tienen asignado este rol
					   list = getRegisters(session, Table.WORKFLOW, "idrol", boRol.getId());
					   
					   System.out.println("WorkFlow con este rol:" +  list.size());
					   
					   // Recorremos todos los usuarios-roles recuperados.
					   for (Object objBOWorkFlow : list)
					   {
						   // Comprobamos que el objeto es de tipo BOWorkFlow.
						   if (objBOWorkFlow instanceof BOWorkFlow)
						   {
							   // Realizamos el cast del objeto
							   BOWorkFlow boWorkFlow = (BOWorkFlow) objBOWorkFlow;
							   
							   
							   //Por cada WorkFlow que tiene este rol debemos de borrar sus estados
							   Set<BOWorkFlow_State> boWorkFlow_States = boWorkFlow.getIdstate_next();
							   
							// Recorremos el conjunto de estados pr&oacute;ximos del WorkSpace
							   for (BOWorkFlow_State boWorkFlow_State : boWorkFlow_States)
								{
								   
								   session.delete(boWorkFlow_State);
								}
							   
							   session.delete(boWorkFlow);
						   }
					   }
					   
					// Borramos rol
					   session.delete(boRol);
						 
					// Commit.
					   session.flush();
					   
				   }
				   		
				}
				else 
				{
				 // Identificador no V&aacute;lido
					throw (new ExceptionControl(ExceptionControl.MSG_PROJECT_ID_INVALID));
				}
			}		
		   catch (Exception ex)
			{
				// Propagamos la excepci&oacute;n
				throw (new ExceptionControl(ex.getMessage()));
			}
			finally
			{
			 // Cerramos la sesi&oacute;n de Hibernate.
				closeSession(session);
			}
		
	}

	
	/**
	 * <p>Este M&eacute;todo es llamado cada vez que se quiere consultar  la informaci&oacute;n de
	 * un rol de un proyecto a partir de la informaci&oacute;n recibida por par&aacute;metro </p>
	 * 
	 * @param idProject <code>int</code> identificador del proyecto.
	 * @param idRol     <code>int</code> identificador del rol del proyecto.
	 * 
	 * @return <code>TORol</code> contiene la informaci&oacute;n del rol cuyo
	 * <i>id</i> es igual al que se ha recibido como par&aacute;metro.
	 * 
	 */
	@Override
	public TORol getRolProject(int idProject, int idRol)
			throws ExceptionControl {
		   
		System.out.println("Abrimos sesi&oacute;n de Hibernate");

		// Abrimos la sesi&oacute;n de Hibernate.
		Session session = this.sessionFactory.openSession();
			
		try
		{
			System.out.println("Comprobamos que el proyecto est&aacute; registrado");
			// Comprobamos que el proyecto est&aacute; registrado
			Object objBOProject = existRegister(session, Table.PROJECT, "id", idProject);
			
			System.out.println("Comprobamos que el rol est&aacute; registrado");
			// Comprobamos que el rol est&aacute; registrado
			Object objBORol = existRegister(session, Table.ROL, "id", idRol);
			
			// Comprobamos que no es nulo y adem&aacute;s es de tipo BOProject.
			// Comprobamos que no es nulo y adem&aacute;s es de tipo BORol.
			if ((objBOProject != null) && (objBOProject instanceof BOProject) &&
				(objBORol != null) && (objBORol instanceof BORol))
				{
					System.out.println("Casteamos a objBOProject");
					// Realizamos el cast del objeto
					BOProject boProject = (BOProject) objBOProject;
					
					System.out.println("Casteamos a objBORol");
					// Realizamos el cast del objeto
					BORol boRol = (BORol) objBORol;
					
					//Verificams que el Rol solicitado es del proyecto que hemos
					//recibido por par&aacute;metro
					if(boRol.getIdproject().getId() != boProject.getId()){
						
						System.out.println("Identificador de Proyecto no coincide con el de rol");
						// Identificador no V&aacute;lido
						throw (new ExceptionControl(ExceptionControl.MSG_PROJECT_ID_INVALID));
					}
					
					System.out.println("Retornamos Datos deL Rol");
					
					 //Retornamos la informaci&oacute;n del objeto en un objeto TORol
					return boRol.getTORol();
				}
				
				System.out.println("Rol no existe");
				 // Identificador no V&aacute;lido
				throw (new ExceptionControl(ExceptionControl.MSG_PROJECT_ID_INVALID));
			}
			finally
			{
				System.out.println("Cerramos sesi&oacute;n");
				// Cerramos la sesi&oacute;n de Hibernate.
				closeSession(session);
			}
	}

	
	/**
	 * <p>Este M&eacute;todo es llamadado cada vez que se quiera dar de alta a un estado dentro de un proyecto
	 * a partir de la informaci&oacute;n recibida por par&aacute;metro.</p>
	 * 
	 * @param state <code>TOState</code> Informaci&oacute;n del Estado de un Projecto.
	 * 
	 * @return <code>TOState</code> contiene la informaci&oacute;n del estado creado</p>
	 * 
	 */
	@Override
	public TOState createStateProject(TOState state)
			throws ExceptionControl {
		
		System.out.println("Entramos en M&eacute;todo createStateProject");
		// Par&aacute;metro nulo
		if (state == null) throw (new ExceptionControl(ExceptionControl.MSG_PROJECT_PARAM_NULL));
		
		System.out.println("Abrimos sesi&oacute;n de Hibernate");
		// Abrimos la sesi&oacute;n de Hibernate.
		Session session = this.sessionFactory.openSession();
		
		try
		{
			
			System.out.println("Verificamos si el proyecto est&aacute; registrado");
			// Comprobamos que el proyecto est&aacute; registrado
			Object objBOProject = existRegister(session, Table.PROJECT, "id", state.getIdproject());
			
			if (objBOProject== null || !(objBOProject instanceof BOProject))
			{
				// Lanzamos Excepci&oacute;n
				throw (new ExceptionControl(ExceptionControl.MSG_PROJECT_ID_INVALID));
				
			}
			
			System.out.println("Verificamos si name del estado registrado para el proyecto");
			// Comprobamos si "name" ya se encuentra registrado para el id del proyecto.
			if (existRegister(session, Table.ROL, "idproject", state.getIdproject(), "name", state.getName()) != null)
			{
				// Lanzamos Excepci&oacute;n
				throw (new ExceptionControl(ExceptionControl.MSG_ROL_NAME_REGISTER));
			}
						
			System.out.println("Casteamos a objBOProject");
			// Realizamos el cast del objeto
			BOProject boProject = (BOProject) objBOProject;
			
			System.out.println("Instanciamos clase BOState");
			// Iniciamos proceso de creaci&oacute;n
			// Creamos un objeto BOState a partir del valor del par&aacute;metro.
			BOState boState = new BOState(boProject,state.getName());
			
			System.out.println("Actualizamos");
			// Grabamos la informaci&oacute;n en B.D.
			session.saveOrUpdate(boState);
			
			System.out.println("Realizamos Commit");
			// Realizamos commit.
			session.flush();
			
			System.out.println("Retornamos Objeto Actualizado");
			// Retornamos la informaci&oacute;n actualizada
			return boState.getTOState();
		}
		finally
		{
			System.out.println("Cerramos Sesion");
			// Cerramos la sesi&oacute;n de Hibernate.
			closeSession(session);
		}
	}

	/**
	 * <p>M&eacute;todo para listar los estados de un proyecto registrado en el sistema</p>
	 * 
	 * @param project <code>TOProject</code> informaci&oacute;n del proyecto que queremos consultar sus estados
	 * 
	 * @return List<TOState> contiene la informaci&oacute;n de los estados definidos en el proyecto
	 * 
	 * @throws ExceptionControl excepci&oacute;n generada por persistencia
	 *  
	 */
	@Override
	public List<TOState> listStateProject(TOProject project)
			throws ExceptionControl {
		
		// Abrimos una sesi&oacute;n de Hibernate.
		Session session = sessionFactory.openSession();
		
		try
		{
			System.out.println("Verificamos si el proyecto est&aacute; registrado");
			// Comprobamos que el proyecto est&aacute; registrado
			Object objBOProject = existRegister(session, Table.PROJECT, "id", project.getId());
			
			if (objBOProject== null || !(objBOProject instanceof BOProject))
			{
				// Lanzamos Excepci&oacute;n
				throw (new ExceptionControl(ExceptionControl.MSG_PROJECT_ID_INVALID));
				
			}
			
			System.out.println("Obtenemos los estados registrados en un proyecto");
			// Obtenemos la lista de todos los estados registrados para un proyecto.
			List<Object> list = getRegisters(session, Table.STATE, "idproject", project.getId());
			
			System.out.println("Construimos la lista de los estados de un proyecto");
			// Construimos la lista con los objetos que ser&aacute;n retornados.
			List<TOState> states = new ArrayList<TOState>();
			
			// Recorremos todos los estados recuperados.
			for (Object objBOState : list)
			{
				// Comprobamos que el objeto es de tipo BOState.
				if (objBOState instanceof BOState)
				{
					// Realizamos el cast del objeto
					BOState boState = (BOState) objBOState;
					// A&ntilde;adimos a la lista el TOState equivalente a BOState.
					states.add(boState.getTOState());
				}
			}
			
			System.out.println("Ordenamos la lista de los estados de un proyecto");
			// Una vez tenemos todos los estados los ordenamos por "name"
			Collections.sort(states);
			
			System.out.println("retornamos la lista de los estados de un proyecto");
			// Retornamos la lista de estados ordenados.
			return states;
		}
		catch (Exception e)
		{
			// Lanzamos Excepci&oacute;n
			throw new ExceptionControl(e.getMessage());
		}
		finally
		{
			// Cerramos la sesi&oacute;n de Hibernate
			closeSession(session);
		}
	}

	/**
	 * <p>Este M&eacute;todo es llamado cada vez que se quiere modificar  la informaci&oacute;n de
	 * un estado de un proyecto a partir de la informaci&oacute;n recibida por par&aacute;metro </p>
	 * 
	 * @param idProject <code>int</code> identificador del proyecto.
	 * @param idState <code>int</code> identificador del estado.
	 * @param nameState <code>String</code> nombre del estado
	 * 
	 * @return <code>TOState</code> informaci&oacute;n del estado actualizada
	 *  
	 */
	@Override
	public TOState modifyStateProject(int idProject, int idState,
			String nameState) throws ExceptionControl {
		
		// Abrimos la sesi&oacute;n de Hibernate.
		Session session = this.sessionFactory.openSession();
					
		try
		{
			// Comprobamos que el proyecto est&aacute; registrado 
			   Object objBOProject = existRegister(session, Table.PROJECT, "id", idProject);
			   
			// Comprobamos que el estado est&aacute; registrado 
			   Object objBOState = existRegister(session, Table.STATE, "id", idState);
			   		   
			// Comprobamos que no es nulo y adem&aacute;s es de tipo BOProject.
			// Comprobamos que no es nulo y adem&aacute;s es de tipo BOState.
			   if ((objBOProject != null) && (objBOProject instanceof BOProject) &&
				   (objBOState != null) && (objBOState instanceof BOState))
					{
						
				   System.out.println("Casteamos a objBOProject");
				   // Realizamos el cast del objeto
				   BOProject boProject = (BOProject) objBOProject;
							
				   System.out.println("Casteamos a objBOState");
				   // Realizamos el cast del objeto
				   BOState boState = (BOState) objBOState;
					
				   //Verificamos que el Rol solicitado es del proyecto que hemos
				   //recibido por par&aacute;metro
				   if(boState.getIdproject().getId() != boProject.getId()){
						
						System.out.println("Identificador de Proyecto no coincide con el de estado");
						// Identificador no V&aacute;lido
						throw (new ExceptionControl(ExceptionControl.MSG_PROJECT_ID_INVALID));
						
				   }
				   
				   //Comprobamos que no existe el name asignado a otro estado en el proyecto
				   if(!boState.getName().equals(nameState) && 
					   existRegister(session, Table.STATE, "name", nameState) != null){
						//El name se encuentra registrado con otro estado en el proyecto
						throw (new ExceptionControl(ExceptionControl.MSG_STATE_NAME_REGISTER));
					}
																				
					// Actualizamos la Informaci&oacute;n del Estado
					boState.setName(nameState.trim());
								
					// Actualizamos la informaci&oacute;n del proyecto.
					session.saveOrUpdate(boState);
					
					// Commit
					session.flush();
					
					// Retornamos un objeto TOState con la informaci&oacute;n actualizada.
					return boState.getTOState();
				}
						   
				// Identificador no V&aacute;lido
				   throw (new ExceptionControl(ExceptionControl.MSG_PROJECT_ID_INVALID));
			}
			catch (Exception ex)
			{
				// Propagamos la excepci&oacute;n
				throw (new ExceptionControl(ex.getMessage()));
			}
			finally
			{
			 // Cerramos la sesi&oacute;n de Hibernate.
				closeSession(session);
			}
	}

	/**
	 * <p>Este M&eacute;todo es llamado cada vez que se quiere dar de baja a un estado de un proyecto
	 * del sistema a partir de la informaci&oacute;n recibida por par&aacute;metro</p>
	 * 
	 * @param idProject <code>int</code> identificador del proyecto.
	 * @param idState   <code>int</code> identificador del estado del proyecto. 
	 * 
	 */
	@Override
	public void deleteStateProject(int idProject, int idState)
			throws ExceptionControl {
		
		// Abrimos la sesi&oacute;n de Hibernate.
		Session session = sessionFactory.openSession();
		   
		try
		{
			// Comprobamos que el proyecto est&aacute; registrado 
			   Object objBOProject = existRegister(session, Table.PROJECT, "id", idProject);
			   
			// Comprobamos que el estado est&aacute; registrado 
			   Object objBOState = existRegister(session, Table.STATE, "id", idState);
			
			// Comprobamos que no es nulo y adem&aacute;s es de tipo BOProject.
			// Comprobamos que no es nulo y adem&aacute;s es de tipo BOState.
			   if ((objBOProject != null) && (objBOProject instanceof BOProject) &&
				   (objBOState != null) && (objBOState instanceof BOState))
					{
					
				   // Realizamos el cast del objeto
				   BOProject boProject = (BOProject) objBOProject;
						
				   // Realizamos el cast del objeto
				   BOState boState = (BOState) objBOState;
				   
				   //Verificamos que el Estado solicitado es del proyecto que hemos
				   //recibido por par&aacute;metro
				   if(boState.getIdproject().getId() != boProject.getId()){
						
						// Identificador no V&aacute;lido
						throw (new ExceptionControl(ExceptionControl.MSG_PROJECT_ID_INVALID));
						
				   }else{

					   // Obtenemos la lista de todos los workspace que existen en el projecto definidos
					   List<Object> list = getRegisters(session, Table.WORKFLOW, "idproject", boState.getIdproject().getId());
					   
					   // SWitch para determinar si el workflow hay que borrarlo o no.
					   boolean borrarWorkFlow = false;
					   
					   // Recorremos todos los workflows recuperados.
					   for (Object objBOWorkFlow : list)
					   {
						   // Comprobamos que el objeto es de tipo BOWorkFlow.
						   if (objBOWorkFlow instanceof BOWorkFlow)
						   {
							   // Realizamos el cast del objeto
							   BOWorkFlow boWorkFlow = (BOWorkFlow) objBOWorkFlow;
							   
							   //Determinamos si hay que borrar el workflow
							   borrarWorkFlow = false;
							   if(boWorkFlow.getIdstate().equals(boState))  borrarWorkFlow = true;
							   
							   //Recuperamos  los estados asignados al WorkFlow
							   Set<BOWorkFlow_State> boWorkFlow_States = boWorkFlow.getIdstate_next();
							   
							   // Recorremos el conjunto de estados pr&oacute;ximos del WorkSpace
							   for (BOWorkFlow_State boWorkFlow_State : boWorkFlow_States)
								{
								   
								   if(borrarWorkFlow || boWorkFlow_State.getIdstate().equals(boState)) session.delete(boWorkFlow_State);
									   
								}
							   
							   if(borrarWorkFlow){
								   // Commit.
								   session.flush();
								  
								   // Borramos boWorkFlow.
								   session.delete(boWorkFlow);
							   }
						   }
					   }
					   
					// Borramos estado
					   session.delete(boState);
						 
					// Commit.
					   session.flush();
					   
				   }
				   		
				}
				else 
				{
				 // Identificador no V&aacute;lido
					throw (new ExceptionControl(ExceptionControl.MSG_PROJECT_ID_INVALID));
				}
			}		
		   catch (Exception ex)
			{
				// Propagamos la excepci&oacute;n
				throw (new ExceptionControl(ex.getMessage()));
			}
			finally
			{
			 // Cerramos la sesi&oacute;n de Hibernate.
				closeSession(session);
			}
	}

	/**
	 * <p>Este M&eacute;todo es llamado cada vez que se quiere consultar  la informaci&oacute;n de
	 * un estado de un proyecto a partir de la informaci&oacute;n recibida por par&aacute;metro </p>
	 * 
	 * @param idProject <code>int</code> identificador del proyecto.
	 * @param idState   <code>int</code> identificador del estado del proyecto.
	 * 
	 * @return <code>TOState</code> contiene la informaci&oacute;n del estado cuyo
	 * <i>id</i> es igual al que se ha recibido como par&aacute;metro.
	 * 
	 */
	@Override
	public TOState getStateProject(int idProject, int idState)
			throws ExceptionControl {
		
		System.out.println("Abrimos sesi&oacute;n de Hibernate");

		// Abrimos la sesi&oacute;n de Hibernate.
		Session session = this.sessionFactory.openSession();
			
		try
		{
			System.out.println("Comprobamos que el proyecto est&aacute; registrado");
			// Comprobamos que el proyecto est&aacute; registrado
			Object objBOProject = existRegister(session, Table.PROJECT, "id", idProject);
			
			System.out.println("Comprobamos que el estado est&aacute; registrado");
			// Comprobamos que el rol est&aacute; registrado
			Object objBOState = existRegister(session, Table.STATE, "id", idState);
			
			// Comprobamos que no es nulo y adem&aacute;s es de tipo BOProject.
			// Comprobamos que no es nulo y adem&aacute;s es de tipo BOState.
			if ((objBOProject != null) && (objBOProject instanceof BOProject) &&
				(objBOState != null) && (objBOState instanceof BOState))
				{
					System.out.println("Casteamos a objBOProject");
					// Realizamos el cast del objeto
					BOProject boProject = (BOProject) objBOProject;
					
					System.out.println("Casteamos a objBOState");
					// Realizamos el cast del objeto
					BOState boState = (BOState) objBOState;
					
					//Verificamos que el Rol solicitado es del proyecto que hemos
					//recibido por par&aacute;metro
					if(boState.getIdproject().getId() != boProject.getId()){
						
						System.out.println("Identificador de Proyecto no coincide con el del estado");
						// Identificador no V&aacute;lido
						throw (new ExceptionControl(ExceptionControl.MSG_PROJECT_ID_INVALID));
					}
					
					System.out.println("Retornamos Datos del Estado");
					
					 //Retornamos la informaci&oacute;n del objeto en un objeto TOState
					return boState.getTOState();
				}
				
				System.out.println("Estado no existe");
				 // Identificador no V&aacute;lido
				throw (new ExceptionControl(ExceptionControl.MSG_PROJECT_ID_INVALID));
			}
			finally
			{
				System.out.println("Cerramos sesi&oacute;n");
				// Cerramos la sesi&oacute;n de Hibernate.
				closeSession(session);
			}
	}

	
	/**
	 * <p>Este M&eacute;todo es llamadado cada vez que se quiera dar de alta a un WorkFlow
	 * dentro de un proyecto a partir de la informaci&oacute;n recibida por par&aacute;metro.</p>
	 * 
	 * @param workflow <code>TOWorkFlow</code> Informaci&oacute;n del workflow.
	 * 
	 * @return <code>TOWorkFlow</code> contiene la informaci&oacute;n del workflow creado</p>
	 * 
	 */
	@Override
	public TOWorkFlow createWorkFlowProject(TOWorkFlow workflow)
			throws ExceptionControl {
		
		System.out.println("Entramos en M&eacute;todo createWorkFlowProject");
		// Par&aacute;metro nulo
		if (workflow == null) throw (new ExceptionControl(ExceptionControl.MSG_PROJECT_PARAM_NULL));
		
		System.out.println("Abrimos sesi&oacute;n de Hibernate");
		// Abrimos la sesi&oacute;n de Hibernate.
		Session session = this.sessionFactory.openSession();
		
		try
		{
			
			System.out.println("Verificamos si el proyecto est&aacute; registrado");
			// Comprobamos que el proyecto est&aacute; registrado
			Object objBOProject = existRegister(session, Table.PROJECT, "id", workflow.getIdproject());
			
			if (objBOProject== null || !(objBOProject instanceof BOProject))
			{
				// Lanzamos Excepci&oacute;n
				throw (new ExceptionControl(ExceptionControl.MSG_PROJECT_ID_INVALID));
				
			}
			
			System.out.println("Verificamos si el rol est&aacute; registrado");
			// Comprobamos que el rol est&aacute; registrado para el proyecto
			Object objBORol = existRegister(session, Table.ROL, "id", workflow.getIdrol(), "idproject", workflow.getIdproject());
						
			if (objBORol== null || !(objBORol instanceof BORol))
			{
				// Lanzamos Excepci&oacute;n
				throw (new ExceptionControl(ExceptionControl.MSG_PROJECT_ROL_NOT_DEFINE));
							
			}
			
			System.out.println("Verificamos si el estado est&aacute; registrado");
			// Comprobamos que el Estado est&aacute; registrado para el proyecto
			Object objBOState = existRegister(session, Table.STATE, "id", workflow.getIdstate(),"idproject", workflow.getIdproject());
						
			if (objBOState == null || !(objBOState instanceof BOState))
			{
				// Lanzamos Excepci&oacute;n
				throw (new ExceptionControl(ExceptionControl.MSG_PROJECT_STATE_NOT_DEFINE));
							
			}
			
			System.out.println("Verificamos si rol + estado esta registrado para el proyecto");
			// Comprobamos si rol + estado ya se encuentra registrado para el id del proyecto.
			if (existRegister(session, Table.WORKFLOW, "idproject", workflow.getIdproject()
					, "idrol", workflow.getIdrol(),"idstate",workflow.getIdstate()) != null)
			{
				// Lanzamos Excepci&oacute;n
				throw (new ExceptionControl(ExceptionControl.MSG_PROJECT_WORKFLOW_EXIST));
			}
						
			System.out.println("Casteamos a objBOProject");
			// Realizamos el cast del objeto
			BOProject boProject = (BOProject) objBOProject;
			
			System.out.println("Casteamos a objBORol");
			// Realizamos el cast del objeto
			BORol boRol = (BORol) objBORol;
			
			System.out.println("Casteamos a objBOState");
			// Realizamos el cast del objeto
			BOState boState = (BOState) objBOState;
			
			System.out.println("Instanciamos clase BOWorkFlow");
			// Iniciamos proceso de creaci&oacute;n
			// Creamos un objeto BOWorkFlow a partir del valor del par&aacute;metro.
			BOWorkFlow boWorkFlow = new BOWorkFlow(boProject, boRol, boState);
			
			System.out.println("Actualizamos");
			// Grabamos la informaci&oacute;n en B.D.
			session.saveOrUpdate(boWorkFlow);
			
			System.out.println("Realizamos Commit");
			// Realizamos commit.
			session.flush();
			
			//Obtenemos un ojeto de tipo toWorkFlow actualizado
			TOWorkFlow toWorkFlow = boWorkFlow.getTOWorkFlow();
			
			System.out.println("Verificamos si llegan informado una lista de estados proximos");
			List<TOState> statesTo = workflow.getIdstate_next();
			
			//Verificamos si adem&aacute;s hay que dar de alta pr&oacute;ximos estados
			if(statesTo != null){
				
				System.out.println("Tratar lista de estados");
				// Recorremos todos los estados que hemos recibidos.
				for (Object objBOStatep : statesTo)
				{
					// Comprobamos que el objeto es de tipo BOState.
					if (objBOStatep instanceof BOState)
					{
						System.out.println("Tratar estado de la lista");
						// Realizamos el cast del objeto
						BOState boStatep = (BOState) objBOStatep;
						//Obtenemos un ojeto de tipo toWorkFlow actualizado
						toWorkFlow = createWorkFlowStateProject(toWorkFlow, boStatep.getTOState());
						
					}
				}		
			}
			
			System.out.println("Retornamos Objeto Actualizado");
			// Retornamos la informaci&oacute;n actualizada
			return toWorkFlow;
		}
		finally
		{
			System.out.println("Cerramos Sesion");
			// Cerramos la sesi&oacute;n de Hibernate.
			closeSession(session);
		}
	}

	/**
	 * <p>Este M&eacute;todo es llamadado cada vez que se quiera dar de alta a un estado dentro de un WorkFlow
	 * de un proyecto a partir de la informaci&oacute;n recibida por par&aacute;metro.</p>
	 * 
	 * @param project <code>TOProject</code> Informaci&oacute;n del projecto.
	 * @param workflow <code>TOWorkFlow</code> Informaci&oacute;n del workflow. 
	 * @param state <code>TOState</code> Informaci&oacute;n del estado nuevo.
	 * 
	 * @return <code>TOWorkFlow</code> contiene la informaci&oacute;n del workflow actualizado con el nuevo estado</p>
	 * 
	 */
	@Override
	public TOWorkFlow createWorkFlowStateProject(TOWorkFlow workflow, TOState state) throws ExceptionControl {
		
		System.out.println("Entramos en M&eacute;todo createWorkFlowStateProject");
		// Par&aacute;metro nulo
		if (workflow == null || state == null) throw (new ExceptionControl(ExceptionControl.MSG_PROJECT_PARAM_NULL));
		
		System.out.println("Abrimos sesi&oacute;n de Hibernate");
		// Abrimos la sesi&oacute;n de Hibernate.
		Session session = this.sessionFactory.openSession();
		
		try
		{
			System.out.println("Verificamos si el proyecto est&aacute; registrado");
			// Comprobamos que el proyecto est&aacute; registrado
			Object objBOProject = existRegister(session, Table.PROJECT, "id", workflow.getIdproject());
			
			if (objBOProject== null || !(objBOProject instanceof BOProject))
			{
				// Lanzamos Excepci&oacute;n
				throw (new ExceptionControl(ExceptionControl.MSG_PROJECT_ID_INVALID));
				
			}
			
			System.out.println("Verificamos si el workflow est&aacute; registrado");
			// Comprobamos que el workflow est&aacute; registrado para el proyecto
			Object objBOWorkFlow = existRegister(session, Table.WORKFLOW, "id", workflow.getId(),"idproject", workflow.getIdproject());
						
			if (objBOWorkFlow== null || !(objBOWorkFlow instanceof BOWorkFlow))
			{
				// Lanzamos Excepci&oacute;n
				throw (new ExceptionControl(ExceptionControl.MSG_PROJECT_WORKFLOW_NOT_DEFINE));
							
			}
			
			System.out.println("Verificamos si el estado est&aacute; registrado");
			// Comprobamos que el Estado est&aacute; registrado para el proyecto
			Object objBOState = existRegister(session, Table.STATE, "id", state.getId(),"idproject", workflow.getIdproject());
						
			if (objBOState == null || !(objBOState instanceof BOState))
			{
				// Lanzamos Excepci&oacute;n
				throw (new ExceptionControl(ExceptionControl.MSG_PROJECT_STATE_NOT_DEFINE));
							
			}
			
			System.out.println("Verificamos si workflow + estado esta registrado para el proyecto");
			// Comprobamos si workflow + estado ya se encuentra registrado.
			if (existRegister(session, Table.WORKFLOWSTATE, "idworkflow", workflow.getId()
					,"idstate", state.getId()) != null)
			{
				// Lanzamos Excepci&oacute;n
				throw (new ExceptionControl(ExceptionControl.MSG_PROJECT_WORKFLOW_STATE_EXIST));
			}
						
			System.out.println("Casteamos a objBOWorkFlow");
			// Realizamos el cast del objeto
			BOWorkFlow boWorkFlow = (BOWorkFlow) objBOWorkFlow;
			
			System.out.println("Casteamos a objBOState");
			// Realizamos el cast del objeto
			BOState boState = (BOState) objBOState;
			
			System.out.println("Instanciamos clase BOWorkFlowState");
			// Iniciamos proceso de creaci&oacute;n
			// Creamos un objeto BOWorkFlow a partir del valor del par&aacute;metro.
			BOWorkFlow_State boWorkFlowState = new BOWorkFlow_State(boWorkFlow, boState);
			
			System.out.println("Actualizamos");
			// Grabamos la informaci&oacute;n en B.D.
			session.saveOrUpdate(boWorkFlowState);
			
			System.out.println("Realizamos Commit");
			// Realizamos commit.
			session.flush();
			
			System.out.println("Retornamos Objeto Actualizado");
			// Retornamos la informaci&oacute;n actualizada
			return getWorkFlowProject(workflow.getIdproject(), workflow.getId());
		}
		finally
		{
			System.out.println("Cerramos Sesion");
			// Cerramos la sesi&oacute;n de Hibernate.
			closeSession(session);
		}
	}
	
	/**
	 * <p>Este m&eacute;todo es llamado cada vez que se quiere consultar la lista de los
	 * workflows definidos en un proyecto.</p>
	 * 
	 * @param project <code>TOProject</code> Informaci&oacute;n del projecto.
	 * 
	 * @return <code>List&lt;TOWorkFlow&gt;</code> lista de worksflows de un Proyecto
	 * 
	 */
	@Override
	public List<TOWorkFlow> listWorkFlowProject(TOProject project)
			throws ExceptionControl {
		
		// Abrimos una sesi&oacute;n de Hibernate.
		Session session = sessionFactory.openSession();
		
		try
		{
			System.out.println("Verificamos si el proyecto est&aacute; registrado");
			// Comprobamos que el proyecto est&aacute; registrado
			Object objBOProject = existRegister(session, Table.PROJECT, "id", project.getId());
			
			if (objBOProject== null || !(objBOProject instanceof BOProject))
			{
				// Lanzamos Excepci&oacute;n
				throw (new ExceptionControl(ExceptionControl.MSG_PROJECT_ID_INVALID));
				
			}
			
			System.out.println("Obtenemos los workflows registrados en un proyecto");
			// Obtenemos la lista de todos los estados registrados para un proyecto.
			List<Object> list = getRegisters(session, Table.WORKFLOW, "idproject", project.getId());
			
			System.out.println("Construimos la lista de los workflows de un proyecto");
			// Construimos la lista con los objetos que ser&aacute;n retornados.
			List<TOWorkFlow> workflows = new ArrayList<TOWorkFlow>();
			
			// Recorremos todos los estados recuperados.
			for (Object objBOWorkFlow : list)
			{
				// Comprobamos que el objeto es de tipo BOWorkFlow.
				if (objBOWorkFlow instanceof BOWorkFlow)
				{
					// Realizamos el cast del objeto
					BOWorkFlow boWorkflow = (BOWorkFlow) objBOWorkFlow;
					
					// A&ntilde;adimos a la lista el TOWorkFlow equivalente a BOWorkFlow.
					workflows.add(boWorkflow.getTOWorkFlow());
				}
			}
			
			System.out.println("Ordenamos la lista de los estados de un proyecto");
			// Una vez tenemos todos los estados los ordenamos 
			Collections.sort(workflows);
			
			System.out.println("retornamos la lista de los workflows de un proyecto");
			// Retornamos la lista de estados ordenados.
			return workflows;
		}
		catch (Exception e)
		{
			// Lanzamos Excepci&oacute;n
			throw new ExceptionControl(e.getMessage());
		}
		finally
		{
			// Cerramos la sesi&oacute;n de Hibernate
			closeSession(session);
		}
	}

	/**
	 * <p>Este M&eacute;todo es llamado cada vez que se quiere modificar  la informaci&oacute;n de
	 * un workflow definido en un proyecto a partir de la informaci&oacute;n recibida por par&aacute;metro </p>
	 * 
	 * @param workflow <code>TOWorkFlow</code> Informaci&oacute;n del workflow.
	 * 
	 * @return <code>TOWorkFlow</code> informaci&oacute;n del workflow actualizada.
	 *  
	 */
	@Override
	public TOWorkFlow modifyWorkFlowProject(TOWorkFlow workflow) throws ExceptionControl {
		// Abrimos la sesi&oacute;n de Hibernate.
		Session session = this.sessionFactory.openSession();
					
		try
		{
			// Comprobamos que el proyecto est&aacute; registrado 
			   Object objBOProject = existRegister(session, Table.PROJECT, "id", workflow.getIdproject());
			   
			// Comprobamos que el workflow est&aacute; registrado
			   Object objBOWorkFlow = existRegister(session, Table.WORKFLOW, "id", workflow.getId());
			   
			//  Comprobamos que el rol est&aacute; registrado  
			   Object objBORol = existRegister(session, Table.ROL, "id", workflow.getIdrol());
			   
			//  Comprobamos que el estado est&aacute; registrado  
			   Object objBOState = existRegister(session, Table.STATE, "id", workflow.getIdstate());
			   
			   		   
			// Comprobamos que no es nulo y adem&aacute;s es de tipo BOProject.
			// Comprobamos que no es nulo y adem&aacute;s es de tipo BOWorkFlow.
			// Comprobamos que no es nulo y adem&aacute;s es de tipo BORol.
			// Comprobamos que no es nulo y adem&aacute;s es de tipo BOState.
			   if ((objBOProject != null) && (objBOProject instanceof BOProject) &&
				   (objBOWorkFlow != null) && (objBOWorkFlow instanceof BOWorkFlow) &&
				   (objBORol != null) && (objBORol instanceof BORol) &&
				   (objBOState != null) && (objBOState instanceof BOState)){
						
				   System.out.println("Casteamos a objBOProject");
				   // Realizamos el cast del objeto
				   BOProject boProject = (BOProject) objBOProject;
							
				   System.out.println("Casteamos a objBOWorkFlow");
				   // Realizamos el cast del objeto
				   BOWorkFlow boWorkFlow = (BOWorkFlow) objBOWorkFlow;
					
				   System.out.println("Casteamos a objBORol");
				   // Realizamos el cast del objeto
				   BORol boRol = (BORol) objBORol;

				   System.out.println("Casteamos a objBOState");
				   // Realizamos el cast del objeto
				   BOState boState = (BOState) objBOState;
				   
				   //Verificamos que el WorkFlow solicitado es del proyecto que hemos
				   //recibido por par&aacute;metro
				   if(boWorkFlow.getIdproject().getId() != boProject.getId()){
						
						System.out.println("Identificador de Proyecto no coincide con el del workflow");
						// Identificador no V&aacute;lido
						throw (new ExceptionControl(ExceptionControl.MSG_PROJECT_ID_INVALID));
				   }
				   
				   System.out.println("Verificamos si rol + estado esta registrado para el proyecto");
					// Comprobamos si rol + estado ya se encuentra registrado para el id del proyecto.
					if (existRegister(session, Table.WORKFLOW, "idproject", workflow.getIdproject()
							, "idrol", workflow.getIdrol(),"idstate",workflow.getIdstate()) != null)
					{
						// Lanzamos Excepci&oacute;n
						throw (new ExceptionControl(ExceptionControl.MSG_PROJECT_WORKFLOW_EXIST));
					}
																				
					// Actualizamos la Informaci&oacute;n del WorkFlow
					boWorkFlow.setIdrol(boRol);
					boWorkFlow.setIdstate(boState);
								
					// Actualizamos la informaci&oacute;n del proyecto.
					session.saveOrUpdate(boWorkFlow);
					
					// Commit
					session.flush();
					
					// Retornamos un objeto TOWorkFlow con la informaci&oacute;n actualizada.
					return boWorkFlow.getTOWorkFlow();
				}
						   
				// Identificador no V&aacute;lido
				   throw (new ExceptionControl(ExceptionControl.MSG_PROJECT_ID_INVALID));
			}
			catch (Exception ex)
			{
				// Propagamos la excepci&oacute;n
				throw (new ExceptionControl(ex.getMessage()));
			}
			finally
			{
			 // Cerramos la sesi&oacute;n de Hibernate.
				closeSession(session);
			}
	}

	/**
	 * <p>Este M&eacute;todo es llamado cada vez que se quiere eliminar a un workflow definido en un proyecto
	 * a partir de la informaci&oacute;n recibida por par&aacute;metro</p>
	 * 
	 * @param idProject <code>int</code> identificador del proyecto. 
	 * @param idWorkFlow <code>int</code> identificador del workflow.
	 * 
	 */
	@Override
	public void deleteWorkFlowProject(int idProject, int idWorkFlow)
			throws ExceptionControl {
		
		// Abrimos la sesi&oacute;n de Hibernate.
		Session session = sessionFactory.openSession();
		   
		try
		{
			// Comprobamos que el proyecto est&aacute; registrado 
			   Object objBOProject  = existRegister(session, Table.PROJECT, "id", idProject);
			   
			// Comprobamos que el workflow est&aacute; registrado 
			   Object objBOWorkFlow = existRegister(session, Table.WORKFLOW, "id", idWorkFlow);
			
			// Comprobamos que no es nulo y adem&aacute;s es de tipo BOProject.
			// Comprobamos que no es nulo y adem&aacute;s es de tipo BOWorkFlow.
			   if ((objBOProject != null) && (objBOProject instanceof BOProject) &&
				   (objBOWorkFlow != null) && (objBOWorkFlow instanceof BOWorkFlow))
					{
					
				   System.out.println("Casteamos a objBOProject");
				   // Realizamos el cast del objeto
				   BOProject boProject = (BOProject) objBOProject;
						
				   System.out.println("Casteamos a objBOWorkFlow");
				   // Realizamos el cast del objeto
				   BOWorkFlow boWorkFlow = (BOWorkFlow) objBOWorkFlow;
				   
				   //Verificamos que el WorkFlow solicitado es del proyecto que hemos
				   //recibido por par&aacute;metro
				   if(boWorkFlow.getIdproject().getId() != boProject.getId()){
						
						System.out.println("Identificador de Proyecto no coincide con el de workflow");
						// Identificador no V&aacute;lido
						throw (new ExceptionControl(ExceptionControl.MSG_PROJECT_ID_INVALID));
						
				   }else{

					   
					//Recuperamos  los estados asignados al WorkFlow
					  Set<BOWorkFlow_State> boWorkFlow_States = boWorkFlow.getIdstate_next();
							   
					// Recorremos el conjunto de estados pr&oacute;ximos del WorkSpace
					  for (BOWorkFlow_State boWorkFlow_State : boWorkFlow_States)
					  {
						  session.delete(boWorkFlow_State);
									   
					   }
							   
					// Commit.
					   session.flush();
								  
					   
					// Borramos workflow
					   session.delete(boWorkFlow);
						 
					// Commit.
					   session.flush();
					   
				   }
				}
				else 
				{
				 // Identificador no V&aacute;lido
					throw (new ExceptionControl(ExceptionControl.MSG_PROJECT_ID_INVALID));
				}
			}		
		   catch (Exception ex)
			{
				// Propagamos la excepci&oacute;n
				throw (new ExceptionControl(ex.getMessage()));
			}
			finally
			{
			 // Cerramos la sesi&oacute;n de Hibernate.
				closeSession(session);
			}
		
	}

	/**
	 * <p>Este M&eacute;todo es llamado cada vez que se quiere eliminar a un estado de un workflow definido en un proyecto
	 * a partir de la informaci&oacute;n recibida por par&aacute;metro</p>
	 * 
	 * @param idProject <code>int</code> identificador del proyecto. 
	 * @param idWorkFlow <code>int</code> identificador del workflow.
	 * @param idState <code>int</code> identificador de un estado del workflow.
	 * 
	 */
	@Override
	public void deleteWorkFlowProject(int idProject, int idWorkFlow,
			int idState) throws ExceptionControl {
		
		// Abrimos la sesi&oacute;n de Hibernate.
			Session session = sessionFactory.openSession();
				   
			try
			{
			 // Comprobamos que el proyecto est&aacute; registrado 
			   Object objBOProject  = existRegister(session, Table.PROJECT, "id", idProject);
					   
			// Comprobamos que el workflow est&aacute; registrado 
			   Object objBOWorkFlow = existRegister(session, Table.WORKFLOW, "id", idWorkFlow);

			// Comprobamos que el estado del workflow est&aacute; registrado 
			   Object objBOWorkFlowState = existRegister(session, Table.WORKFLOWSTATE, "idworkflow", idWorkFlow,"idstate",idState);
 			   
			// Comprobamos que no es nulo y adem&aacute;s es de tipo BOProject.
			// Comprobamos que no es nulo y adem&aacute;s es de tipo BOWorkFlow.
			   if ((objBOProject != null) && (objBOProject instanceof BOProject) &&
				   (objBOWorkFlow != null) && (objBOWorkFlow instanceof BOWorkFlow) &&   
				   (objBOWorkFlowState != null) && (objBOWorkFlowState instanceof BOWorkFlow_State))
					{
							
				   System.out.println("Casteamos a objBOProject");
				   // Realizamos el cast del objeto
				   BOProject boProject = (BOProject) objBOProject;
								
				   System.out.println("Casteamos a objBOWorkFlow");
				   // Realizamos el cast del objeto
				   BOWorkFlow boWorkFlow = (BOWorkFlow) objBOWorkFlow;

				   System.out.println("Casteamos a objBOWorkFlowState");
				   // Realizamos el cast del objeto
				   BOWorkFlow_State boWorkFlowState = (BOWorkFlow_State) objBOWorkFlowState;
				   
				   //Verificamos que el WorkFlow solicitado es del proyecto que hemos
				   //recibido por par&aacute;metro 
				   if((boWorkFlow.getIdproject().getId() != boProject.getId())){
								
					   System.out.println("Identificador de Proyecto no coincide con el de workflow");
						// Identificador no V&aacute;lido
							throw (new ExceptionControl(ExceptionControl.MSG_PROJECT_ID_INVALID));
								
					}else{

						// Borramos workflow
						   session.delete(boWorkFlowState);
								 
						// Commit.
						   session.flush();
							   
					}
						   		
				}
				else 
				{
					// Identificador no V&aacute;lido
					throw (new ExceptionControl(ExceptionControl.MSG_PROJECT_ID_INVALID));
				}
			}		
			catch (Exception ex)
			{
				// Propagamos la excepci&oacute;n
				throw (new ExceptionControl(ex.getMessage()));
			}
			finally
			{
				// Cerramos la sesi&oacute;n de Hibernate.
				closeSession(session);
			}
	}
	
	/**
	 * <p>Este M&eacute;todo es llamado cada vez que se quiere consultar  la informaci&oacute;n de
	 * un workflow definido en un proyecto a partir de la informaci&oacute;n recibida por par&aacute;metro</p>
	 * 
	 * @param idProject <code>int</code> identificador del proyecto.
	 * @param idWorkFlow <code>int</code> identificador del workflow.	  
	 * 
	 * @return <code>TOWorkFlow</code> contiene la informaci&oacute;n del workflow cuyo
	 * <i>id</i> es igual al que se ha recibido como par&aacute;metro.
	 * 
	 */
	@Override
	public TOWorkFlow getWorkFlowProject(int idProject, int idWorkFlow)
			throws ExceptionControl {

		// Abrimos la sesi&oacute;n de Hibernate.
		Session session = sessionFactory.openSession();
		   
		try
		{
			System.out.println("idProject=" + idProject);
			System.out.println("idWorkFlow=" + idWorkFlow);
			
			// Comprobamos que el proyecto est&aacute; registrado 
			   Object objBOProject  = existRegister(session, Table.PROJECT, "id", idProject);
			   
			// Comprobamos que el workflow est&aacute; registrado 
			   Object objBOWorkFlow = existRegister(session, Table.WORKFLOW, "id", idWorkFlow);
						
			// Comprobamos que no es nulo y adem&aacute;s es de tipo BOProject.
			// Comprobamos que no es nulo y adem&aacute;s es de tipo BOWorkFlow.
			   if ((objBOProject != null) && (objBOProject instanceof BOProject) &&
				   (objBOWorkFlow != null) && (objBOWorkFlow instanceof BOWorkFlow))
					{
					
				   System.out.println("Casteamos a objBOProject");
				   // Realizamos el cast del objeto
				   BOProject boProject = (BOProject) objBOProject;
						
				   System.out.println("Casteamos a objBOWorkFlow");
				   // Realizamos el cast del objeto
				   BOWorkFlow boWorkFlow = (BOWorkFlow) objBOWorkFlow;
				   
				   
				   System.out.println("Estados definidos en el workspace=" + boWorkFlow.getIdstate_next().size());
				   
				   //Verificamos que el WorkFlow solicitado es del proyecto que hemos
				   //recibido por par&aacute;metro
				   if(boWorkFlow.getIdproject().getId() != boProject.getId()){
						
						System.out.println("Identificador de Proyecto no coincide con el de workflow");
						// Identificador no V&aacute;lido
						throw (new ExceptionControl(ExceptionControl.MSG_PROJECT_ID_INVALID));
						
				   }
				   
				   //Retornamos la informaci&oacute;n del objeto en un objeto TOWorkFlow
					return boWorkFlow.getTOWorkFlowStateNext();
				}
				else 
				{
				 // Identificador no V&aacute;lido
					throw (new ExceptionControl(ExceptionControl.MSG_PROJECT_ID_INVALID));
				}
			}		
		   catch (Exception ex)
			{
				// Propagamos la excepci&oacute;n
				throw (new ExceptionControl(ex.getMessage()));
			}
			finally
			{
			 // Cerramos la sesi&oacute;n de Hibernate.
				closeSession(session);
			}
	}

	/**
	 * <p>Este M&eacute;todo es llamadado cada vez que se quiera dar de alta una dupla usuario-rol
	 * dentro de un proyecto a partir de la informaci&oacute;n recibida por par&aacute;metro.</p>
	 * 
	 * @param project <code>TOProject</code> Informaci&oacute;n del projecto.
	 * @param user <code>TOUser</code> Informaci&oacute;n del usuario.
	 * @param rol <code>TORol</code> Informaci&oacute;n del rol.
	 * 
	 * @return <code>TOUserRol</code> contiene la informaci&oacute;n del workflow creado</p>
	 * 
	 */
	@Override
	public TOUserRol createUserRolProject(TOUserRol userrol) throws ExceptionControl {
		
		System.out.println("Entramos en M&eacute;todo createUserRolProject");
		// Par&aacute;metro nulo
		if (userrol == null) throw (new ExceptionControl(ExceptionControl.MSG_PROJECT_PARAM_NULL));
		
		System.out.println("Abrimos sesi&oacute;n de Hibernate");
		// Abrimos la sesi&oacute;n de Hibernate.
		Session session = this.sessionFactory.openSession();
		
		try
		{
			System.out.println("Verificamos si el proyecto est&aacute; registrado");
			// Comprobamos que el proyecto est&aacute; registrado
			Object objBOProject = existRegister(session, Table.PROJECT, "id", userrol.getIdproject());
			
			if (objBOProject== null || !(objBOProject instanceof BOProject))
			{
				// Lanzamos Excepci&oacute;n
				throw (new ExceptionControl(ExceptionControl.MSG_PROJECT_ID_INVALID));
				
			}
			
			System.out.println("Verificamos si el user est&aacute; registrado");
			// Comprobamos que el user est&aacute; registrado en el sistema
			Object objBOUser = existRegister(session, Table.USER, "id", userrol.getIduser());
						
			if (objBOUser== null || !(objBOUser instanceof BOUser))
			{
				// Lanzamos Excepci&oacute;n
				throw (new ExceptionControl(ExceptionControl.MSG_USER_ID_INVALID));
							
			}
			
			System.out.println("Verificamos si elrol est&aacute; registrado");
			// Comprobamos que el Estado est&aacute; registrado para el proyecto
			Object objBORol = existRegister(session, Table.ROL, "id", userrol.getIdrol(),"idproject", userrol.getIdproject());
						
			if (objBORol == null || !(objBORol instanceof BORol))
			{
				// Lanzamos Excepci&oacute;n
				throw (new ExceptionControl(ExceptionControl.MSG_PROJECT_ROL_NOT_DEFINE));
							
			}
			
			System.out.println("Verificamos si user + rol esta registrado para el proyecto");
			// Comprobamos si user + estado ya se encuentra registrado.
			if (existRegister(session, Table.USERROL, "iduser", userrol.getIduser()
					,"idrol", userrol.getIdrol(), "idproject", userrol.getIdproject()) != null)
			{
				// Lanzamos Excepci&oacute;n
				throw (new ExceptionControl(ExceptionControl.MSG_PROJECT_USER_ROL_EXIST));
			}
						
			System.out.println("Casteamos a objBOUser");
			// Realizamos el cast del objeto
			BOUser boUser = (BOUser) objBOUser;
			
			System.out.println("Casteamos a objBORol");
			// Realizamos el cast del objeto
			BORol boRol = (BORol) objBORol;

			System.out.println("Casteamos a objBOProject");
			// Realizamos el cast del objeto
			BOProject boProject = (BOProject) objBOProject;
			
			System.out.println("Instanciamos clase BOUserRol");
			// Iniciamos proceso de creaci&oacute;n
			// Creamos un objeto BOUserRol a partir del valor de los par&aacute;metros.
			BOUserRol boUserRol = new BOUserRol(boUser, boRol, boProject);
			
			System.out.println("Actualizamos");
			// Grabamos la informaci&oacute;n en B.D.
			session.saveOrUpdate(boUserRol);
			
			System.out.println("Realizamos Commit");
			// Realizamos commit.
			session.flush();
			
			System.out.println("Retornamos Objeto Actualizado");
			// Retornamos la informaci&oacute;n actualizada
			return boUserRol.getTOUserRol();
		}
		finally
		{
			System.out.println("Cerramos Sesion");
			// Cerramos la sesi&oacute;n de Hibernate.
			closeSession(session);
		}
	}

	/**
	 * <p>Este m&eacute;todo es llamado cada vez que se quiere consultar la lista de las duplas usuarios-rol
	 * definidos en un proyecto.</p>
	 * 
	 * @param project <code>TOProject</code> Informaci&oacute;n del projecto.
	 * 
	 * @return <code>List&lt;TOUserRol&gt;</code> lista de duplas usuarios-rol de un Proyecto
	 * 
	 */
	@Override
	public List<TOUserRol> listUserRolProject(TOProject project)
			throws ExceptionControl {
		
		// Abrimos una sesi&oacute;n de Hibernate.
		Session session = sessionFactory.openSession();
		
		try
		{
			System.out.println("Verificamos si el proyecto est&aacute; registrado");
			// Comprobamos que el proyecto est&aacute; registrado
			Object objBOProject = existRegister(session, Table.PROJECT, "id", project.getId());
			
			if (objBOProject== null || !(objBOProject instanceof BOProject))
			{
				// Lanzamos Excepci&oacute;n
				throw (new ExceptionControl(ExceptionControl.MSG_PROJECT_ID_INVALID));
				
			}
			
			System.out.println("Obtenemos los usuarios y sus roles registrados en un proyecto");
			// Obtenemos la lista de todos los estados registrados para un proyecto.
			List<Object> list = getRegisters(session, Table.USERROL, "idproject", project.getId());
			
			System.out.println("Construimos la lista de los userrols de un proyecto");
			// Construimos la lista con los objetos que ser&aacute;n retornados.
			List<TOUserRol> userrols = new ArrayList<TOUserRol>();
			
			// Recorremos todos los usuarios-roles recuperados.
			for (Object objBOUserRol : list)
			{
				// Comprobamos que el objeto es de tipo BOUserRol.
				if (objBOUserRol instanceof BOUserRol)
				{
					// Realizamos el cast del objeto
					BOUserRol boUserRol = (BOUserRol) objBOUserRol;
					
					// A&ntilde;adimos a la lista el TOBoUserRol equivalente a BOUserRol.
					userrols.add(boUserRol.getTOUserRol());
				}
			}
			
			System.out.println("Ordenamos la lista de los usuarios-roles de un proyecto");
			// Una vez tenemos todos los estados los ordenamos 
			Collections.sort(userrols);
			
			System.out.println("retornamos la lista de los usuarios-roles de un proyecto");
			// Retornamos la lista de estados ordenados.
			return userrols;
		}
		catch (Exception e)
		{
			// Lanzamos Excepci&oacute;n
			throw new ExceptionControl(e.getMessage());
		}
		finally
		{
			// Cerramos la sesi&oacute;n de Hibernate
			closeSession(session);
		}
	}
	

	/**
	 * <p>Este M&eacute;todo es llamado cada vez que se quiere eliminar una dupla usuario-rol en un proyecto
	 * a partir de la informaci&oacute;n recibida por par&aacute;metro</p>
	 * 
	 * @param idProject <code>int</code> identificador del proyecto. 
	 * @param idUserRol <code>int</code> identificador de la dupla usuario-rol.
	 * 
	 */
	@Override
	public void deleteUserRolProject(int idProject, int idUserRol)
			throws ExceptionControl {
		
		// Abrimos la sesi&oacute;n de Hibernate.
		Session session = sessionFactory.openSession();
		   
		try
		{
			// Comprobamos que el proyecto est&aacute; registrado 
			   Object objBOProject  = existRegister(session, Table.PROJECT, "id", idProject);
			   
			// Comprobamos que el usrrol est&aacute; registrado 
			   Object objBOUserRol = existRegister(session, Table.USERROL, "id", idUserRol);
			
			// Comprobamos que no es nulo y adem&aacute;s es de tipo BOProject.
			// Comprobamos que no es nulo y adem&aacute;s es de tipo BOUserRol.
			   if ((objBOProject != null) && (objBOProject instanceof BOProject) &&
				   (objBOUserRol != null) && (objBOUserRol instanceof BOUserRol))
					{
					
				   System.out.println("Casteamos a objBOProject");
				   // Realizamos el cast del objeto
				   BOProject boProject = (BOProject) objBOProject;
						
				   System.out.println("Casteamos a objBOUserRol");
				   // Realizamos el cast del objeto
				   BOUserRol boUserRol = (BOUserRol) objBOUserRol;
				   
				   //Verificamos que el UserRol solicitado es del proyecto que hemos
				   //recibido por par&aacute;metro
				   if(boUserRol.getIdproject().getId() != boProject.getId()){
						
						System.out.println("Identificador de Proyecto no coincide con el de user-rol");
						// Identificador no V&aacute;lido
						throw (new ExceptionControl(ExceptionControl.MSG_PROJECT_ID_INVALID));
						
				   }else{

					// Borramos workflow
					   session.delete(boUserRol);
						 
					// Commit.
					   session.flush();
					   
				   }
				   		
				}
				else 
				{
				 // Identificador no V&aacute;lido
					throw (new ExceptionControl(ExceptionControl.MSG_PROJECT_ID_INVALID));
				}
			}		
		   catch (Exception ex)
			{
				// Propagamos la excepci&oacute;n
				throw (new ExceptionControl(ex.getMessage()));
			}
			finally
			{
			 // Cerramos la sesi&oacute;n de Hibernate.
				closeSession(session);
			}
	}

	/**
	 * <p>Este M&eacute;todo es llamado cada vez que se quiere consultar  la informaci&oacute;n de
	 * una dupla usuario-rol definida en un proyecto a partir de la informaci&oacute;n recibida por par&aacute;metro</p>
	 * 
	 * @param idProject <code>int</code> identificador del proyecto.
	 * @param idUserRol <code>int</code> identificador de la dupla usuario-rol.	  
	 * 
	 * @return <code>TOUserRol</code> contiene la informaci&oacute;n de la dupla usuario-rol cuyo
	 * <i>id</i> es igual al que se ha recibido como par&aacute;metro.
	 * 
	 */
	@Override
	public TOUserRol getUserRolProject(int idProject, int idUserR)
			throws ExceptionControl {
		
		// Abrimos la sesi&oacute;n de Hibernate.
		Session session = sessionFactory.openSession();
		   
		try
		{
			// Comprobamos que el proyecto est&aacute; registrado 
			   Object objBOProject  = existRegister(session, Table.PROJECT, "id", idProject);
			   
			// Comprobamos que el usuario-rol est&aacute; registrado 
			   Object objBOUserRol = existRegister(session, Table.USERROL, "id", idUserR);
			
			// Comprobamos que no es nulo y adem&aacute;s es de tipo BOProject.
			// Comprobamos que no es nulo y adem&aacute;s es de tipo BOUSERROL.
			   if ((objBOProject != null) && (objBOProject instanceof BOProject) &&
				   (objBOUserRol != null) && (objBOUserRol instanceof BOUserRol))
					{
					
				   System.out.println("Casteamos a objBOProject");
				   // Realizamos el cast del objeto
				   BOProject boProject = (BOProject) objBOProject;
						
				   System.out.println("Casteamos a objBOUserRol");
				   // Realizamos el cast del objeto
				   BOUserRol boUserRol = (BOUserRol) objBOUserRol;
				   
				   //Verificamos que el UserRol solicitado es del proyecto que hemos
				   //recibido por par&aacute;metro
				   if(boUserRol.getIdproject().getId() != boProject.getId()){
						
						System.out.println("Identificador de Proyecto no coincide con el de user-rol");
						// Identificador no V&aacute;lido
						throw (new ExceptionControl(ExceptionControl.MSG_PROJECT_ID_INVALID));
						
				   }
				   
				   //Retornamos la informaci&oacute;n del objeto en un objeto TOBoUserRol
					return boUserRol.getTOUserRol();
				}
				else 
				{
				 // Identificador no V&aacute;lido
					throw (new ExceptionControl(ExceptionControl.MSG_PROJECT_ID_INVALID));
				}
			}		
		   catch (Exception ex)
			{
				// Propagamos la excepci&oacute;n
				throw (new ExceptionControl(ex.getMessage()));
			}
			finally
			{
			 // Cerramos la sesi&oacute;n de Hibernate.
				closeSession(session);
			}
	}

	@Override
	public TOProject createProjectFrom(TOProject project, int idProjectFrom)
			throws ExceptionControl {
		
		// Par&aacute;metro nulo
		if (project == null) throw (new ExceptionControl(ExceptionControl.MSG_PROJECT_PARAM_NULL));
				
		// Abrimos la sesi&oacute;n de Hibernate.
		Session session = this.sessionFactory.openSession();
				
		try
		{
			System.out.println("Verificamos si prefix_code del proyecto registrado");
			// Comprobamos si "prefix_code" ya se encuentra registrado.
			if (existRegister(session, Table.PROJECT, "prefix_code", project.getPrefix_code()) != null)
			{
				// Lanzamos Excepci&oacute;n
					throw (new ExceptionControl(ExceptionControl.MSG_PROJECT_PREFIX_CODE_REGISTER));
			}
				
			System.out.println("Verificamos si name del proyecto registrado");
			// Comprobamos si "name" ya se encuentra registrado.
			if (existRegister(session, Table.PROJECT, "name", project.getName()) != null)
			{
				// Lanzamos Excepci&oacute;n
				throw (new ExceptionControl(ExceptionControl.MSG_PROJECT_NAME_REGISTER));
			}
			
			System.out.println("Verificamos si idProjectFrom se encuentra registrado");
			// Comprobamos si "idProjectFrom" ya se encuentra registrado.
			Object projectFrom = existRegister(session, Table.PROJECT, "id", idProjectFrom);
			if (projectFrom == null)
			{
				// Lanzamos Excepci&oacute;n
				throw (new ExceptionControl(ExceptionControl.MSG_PROJECT_ID_INVALID));
			}
					
			System.out.println("Instanciamos clase BOProject");
			// Iniciamos proceso de creaci&oacute;n
			// Creamos un objeto BOProject a partir del valor del par&aacute;metro.
			BOProject boProject = new BOProject(project);
					
			System.out.println("Actualizamos");
			// Grabamos la informaci&oacute;n en B.D.
			session.saveOrUpdate(boProject);
					
			System.out.println("Realizamos Commit");
			// Realizamos commit.
			session.flush();
			
			
			System.out.println("idprojectTo  ="+ boProject.getId());
			System.out.println("idprojectFrom="+ idProjectFrom);
			
			System.out.println("Alta de Roles");
			//Damos de alta los roles del Proyecto From
			List<Object> list = getRegisters(session, Table.ROL, "idproject", idProjectFrom); 
			
			// Recorremos todos los roles recuperados.
			 for (Object objBORol : list)
			 {
			   // Comprobamos que el objeto es de tipo BORol.
			   if (objBORol instanceof BORol)
			   {
				   // Realizamos el cast del objeto
				   BORol boRolfrom = (BORol) objBORol;
				   
				   BORol boRol = new BORol(boProject, boRolfrom.getName());
				   
				   session.saveOrUpdate(boRol);
			   }
			 }
			 
			// Realizamos commit.
			session.flush();
			
			System.out.println("Alta de Estados");
			//Damos de alta los estados del Proyecto From
			list = getRegisters(session, Table.STATE, "idproject", idProjectFrom); 
			
			// Recorremos todos los estados recuperados.
			 for (Object objBOState : list)
			 {
			   // Comprobamos que el objeto es de tipo BOState.
			   if (objBOState instanceof BOState)
			   {
				   // Realizamos el cast del objeto
				   BOState boStatefrom = (BOState) objBOState;
				   
				   BOState boState = new BOState(boProject, boStatefrom.getName());
				   
				   session.saveOrUpdate(boState);
			   }
			 }
			
			// Realizamos commit.
			session.flush();
			
			System.out.println("Alta de WorkFlows");
			//Damos de alta los workflows del Proyecto From
			list = getRegisters(session, Table.WORKFLOW, "idproject", idProjectFrom); 
			
			// Recorremos todos los workflows recuperados.
			 for (Object objBOWorkFlow : list)
			 {
			   // Comprobamos que el objeto es de tipo BOWorkFlow.
			   if (objBOWorkFlow instanceof BOWorkFlow)
			   {
				   // Realizamos el cast del objeto
				   BOWorkFlow boWorkFlowfrom = (BOWorkFlow) objBOWorkFlow;
				   
				   //Recuperamos el idRol
				   Object ObjRol = existRegister(session, Table.ROL, "idproject", boProject.getId(),"name",boWorkFlowfrom.getIdrol().getName());
				   //Casteamos el objeto recuperado
				   BORol boRol = (BORol) ObjRol;
				   
				   //Recuperamos el idState
				   Object ObjState = existRegister(session, Table.STATE, "idproject", boProject.getId(),"name",boWorkFlowfrom.getIdstate().getName());
				   //Casteamos el objeto recuperado
				   BOState boState = (BOState) ObjState;
				   
				   //Creamos un nuevo WorkFlow
				   BOWorkFlow boWorkFlow = new BOWorkFlow(boProject, boRol, boState);
				   
				   session.saveOrUpdate(boWorkFlow);
				   
				// Realizamos commit.
				   session.flush();
				   
				   System.out.println("Alta de Estados de WorkFlows");
				 //Recuperamos  los estados asignados al WorkFlow
				   Set<BOWorkFlow_State> boWorkFlow_States = boWorkFlowfrom.getIdstate_next();
							   
				 // Recorremos el conjunto de estados pr&oacute;ximos del WorkSpace
				   for (BOWorkFlow_State boWorkFlow_StateFrom : boWorkFlow_States)
				   {
					
					     //Recuperamos el idState
					   	 Object ObjStateN = existRegister(session, Table.STATE, "idproject", boProject.getId(),"name",boWorkFlow_StateFrom.getIdstate().getName());
					   	 //Casteamos el objeto recuperado
						 BOState boStateN = (BOState) ObjStateN;
					   
						//Creamos un nuevo WorkFlow_State
						 BOWorkFlow_State boWorkFlow_State = new BOWorkFlow_State(boWorkFlow, boStateN); 
					   
						 session.saveOrUpdate(boWorkFlow_State);
									   
					}
							   
				 // Commit.
				   session.flush();
			   }
			 }
			
			 System.out.println("Alta de User-Rols");
			//Damos de alta los roles-usuarios del Proyecto From
			  list = getRegisters(session, Table.USERROL, "idproject", idProjectFrom); 
				
			// Recorremos todos los usuarios-roles recuperados.
			   for (Object objBOUserRol : list)
				 {
				   // Comprobamos que el objeto es de tipo BOUserRol.
				   if (objBOUserRol instanceof BOUserRol)
				   {
					   // Realizamos el cast del objeto
					   BOUserRol boUserRolfrom = (BOUserRol) objBOUserRol;
					   
					  //Recuperamos el idUser
					   Object ObjUser = existRegister(session, Table.USER, "id",boUserRolfrom.getIduser().getId());
					   //Casteamos el objeto recuperado
					   BOUser boUser = (BOUser) ObjUser;
					   
					 //Recuperamos el idRol
					   Object ObjRol = existRegister(session, Table.ROL, "idproject", boProject.getId(),"name",boUserRolfrom.getIdrol().getName());
					   //Casteamos el objeto recuperado
					   BORol boRol = (BORol) ObjRol;
					   
					   BOUserRol boUserRol = new BOUserRol(boUser, boRol, boProject);
					   
					   session.saveOrUpdate(boUserRol);
				   }
				 }
				
				// Realizamos commit.
				session.flush();
					
				System.out.println("Retornamos Objeto Actualizado");
				// Retornamos la informaci&oacute;n actualizada
				return boProject.getTOProject();
			}
			finally
			{
				System.out.println("Cerramos Sesion");
				// Cerramos la sesi&oacute;n de Hibernate.
				closeSession(session);
			}
	}
		
}
