package edu.uoc.integra.business.functions;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import org.hibernate.Session;

import edu.uoc.integra.business.exceptions.ExceptionControl;
import edu.uoc.integra.business.interfaces.IBUserManager;
import edu.uoc.integra.business.objects.BOConfig;
import edu.uoc.integra.business.objects.BOUser;
import edu.uoc.integra.business.objects.BOUserRol;
import edu.uoc.integra.business.objects.transfers.TOUser;



/**
 * <p>Implementa las funcionalidades ofrecidas por la interfaz <code>IBUserManager</code> </p>
 * 
 * @author Miguel Angel Pineda Cruz
 * @version 1.0 
 * @see IBUserManager
 */
public class IBUserManagerImpl extends IBCommonManagerImpl implements IBUserManager 
{
	
	//************************************************************//
	// CONSTRUCTOR.                                               //
	//************************************************************//
	/**
	 * <p><code>Construye un nuevo objeto <code>IBUserManagerImpl</code> e inicializa el 
	 * marco de persistencia <i>Hibernate</i>.</p>
	 * @throws ExceptionControl error al iniciar el marco de persistencia.
	 */
	public IBUserManagerImpl() throws ExceptionControl
	{
		// Constructor de la superclase.
		super();
	}

	
	/**
	 * <p>Este M&eacute;todo es llamadado cada vez que se quiera dar de alta a un usuario
	 * a partir de la informaci&oacute;n recibida por par&aacute;metro.</p>
	 * 
	 * @param user <code>TOUser</code> Informaci&oacute;n del Usuario a crear.
	 * 
	 * @return <code>TOUser</code> contiene la informaci&oacute;n del usuario creado</p>
	 * 
	 */
	@Override
	public TOUser createUser(TOUser user) throws ExceptionControl {
		
		System.out.println("Entramos en M&eacute;todo createUser");
		// Par&aacute;metro nulo
		if (user == null) throw (new ExceptionControl(ExceptionControl.MSG_USER_PARAM_NULL));
		
		
		System.out.println("Abrimos sesi&oacute;n de Hibernate");
		// Abrimos la sesi&oacute;n de Hibernate.
		Session session = this.sessionFactory.openSession();
		
		try
		{
			System.out.println("Verificamos si login registrado");
			// Comprobamos si "login" ya se encuentra registrado.
			if (existRegister(session, Table.USER, "login", user.getLogin()) != null)
			{
				// Lanzamos Excepci&oacute;n
				throw (new ExceptionControl(ExceptionControl.MSG_USER_LOGIN_REGISTER));
			}
			
			System.out.println("Verificamos si email registrado");
			// Comprobamos si "email" ya se encuentra registrado.
			if (existRegister(session, Table.USER, "email", user.getEmail()) != null)
			{
				// Lanzamos Excepci&oacute;n
				throw (new ExceptionControl(ExceptionControl.MSG_USER_EMAIL_REGISTER));
			}
			
			System.out.println("Instanciamos clase BOUser");
			// Iniciamos proceso de creaci&oacute;n
			// Creamos un objeto BOUser a partir del valor del par&aacute;metro.
			BOUser boUser = new BOUser(user);
			
			System.out.println("Actualizamos");
			// Grabamos la informaci&oacute;n en B.D.
			session.saveOrUpdate(boUser);
			
			System.out.println("Realizamos Commit");
			// Realizamos commit.
			session.flush();
			
			System.out.println("Retornamos Objeto Actualizado");
			// Retornamos la informaci&oacute;n actualizada
			return boUser.getTOUser();
		}
		finally
		{
			System.out.println("Cerramos Sesion");
			// Cerramos la sesi&oacute;n de Hibernate.
			closeSession(session);
		}
	}

		
	/**
	 * <p>Este m&eacute;todo es llamado cada vez que se quiere consultar la lista de
	 * usuarios registrados en el sistema.</p>
	 * 
	 * @return <code>List<TOUser></code> lista de usuarios
	 * 
	 */
	@Override
	public List<TOUser> listUser() throws ExceptionControl {

		
		System.out.println("Abrimos sesi&oacute;n de Hibernate");
		
		// Abrimos una sesi&oacute;n de Hibernate.
		Session session = sessionFactory.openSession();
		
		try
		{
			System.out.println("Recuperamos todos los registros");
			
			// Obtenemos la lista de todos los usuarios registrados.
			List<Object> list = getRegisters(session, Table.USER);
			
			System.out.println("Construimos lista");
			
			// Construimos la lista con los objetos que ser&aacute;n retornados.
			List<TOUser> users = new ArrayList<TOUser>();
			
			// Recorremos todos los usuarios.
			for (Object objBOUser : list)
			{
				// Comprobamos que el objeto es de tipo BOUser.
				if (objBOUser instanceof BOUser)
				{
					// Realizamos el cast del objeto
					BOUser boUser = (BOUser) objBOUser;
					
					// A&ntilde;adimos a la lista el TOUser equivalente a BOUser.
					users.add(boUser.getTOUser());
				}
			}
			
			System.out.println("Ordenamos lista");
			// Una vez tenemos todos los usuarios los ordenamos por "id"
			Collections.sort(users);
			
			// Retornamos la lista de usuarios.
			return users;
		}
		catch (Exception e)
		{
			
			System.out.println("Se ha producido un error:" + e.getMessage());
			// Lanzamos Excepci&oacute;n
			throw new ExceptionControl(e.getMessage());
		}
		finally
		{
			System.out.println("Cerramos lista");
			
			// Cerramos la sesi&oacute;n de Hibernate
			closeSession(session);
		}
	}

	
	/**
	 * <p>M&eacute;todo para listar los usuarios registrados en el sistema que cumplan los criterios que se han
	 * pasado por par&aacute;metro</p>
	 * 
	 * @param login <code>String</code> usuario que contiene este login
	 * @param company <code>String</code> usuario que contiene esta company
	 * @param email <code>String</code> usuario que contiene este email
	 * @param type <code>String</code> usuario que contiene este tipo de autorizaci&oacute;n
	 * 
	 * @return List<TOUser> contiene la informaci&oacute;n de los usuarios que cumplen los criterios
	 * 
	 * @throws ExceptionControl excepci&oacute;n generada por persistencia
	 *  
	 */
	@Override
	public List<TOUser> listUser(String login, String company, String email,
			String type) throws ExceptionControl {

		// Abrimos una sesi&oacute;n de Hibernate.
		Session session = sessionFactory.openSession();
		
		try
		{
			// Obtenemos la lista de todos los usuarios registrados.
			List<Object> list = getRegisters(session, Table.USER);
			
			// Construimos la lista con los objetos que ser&aacute;n retornados.
			List<TOUser> users = new ArrayList<TOUser>();
			
			// Recorremos todos los usuarios.
			for (Object objBOUser : list)
			{
				// Comprobamos que el objeto es de tipo BOUser.
				if (objBOUser instanceof BOUser)
				{
					// Realizamos el cast del objeto
					BOUser boUser = (BOUser) objBOUser;
					
					System.out.println("login=" + login + "#");
					System.out.println("company=" + company + "#");
					System.out.println("email=" + email + "#");
					System.out.println("type=" + type + "#");
					
					
					
					// Comprobamos si el objeto recuperado cumple con
					// las condiciones prefijadas por par&aacute;metro
					if(boUser.getLogin().contains(login) &&
					   boUser.getCompany().contains(company) &&
					   boUser.getEmail().contains(email) &&
					   boUser.getType().contains(type))
					  // A&ntilde;adimos a la lista el TOUser equivalente a BOUser.
						users.add(boUser.getTOUser());
				}
			}
			
			// Una vez tenemos todos los usuarios los ordenamos por "id"
			Collections.sort(users);
			
			// Retornamos la lista de usuarios filtrados.
			return users;
		}
		catch (Exception e)
		{
			// Lanzamos Excepci&oacute;n
			throw new ExceptionControl(e.getMessage());
		}
		finally
		{
			// Cerramos la sesi&oacute;n de Hibernate
			closeSession(session);
		}
	}
		
	/**
	 * <p>Este M&eacute;todo es llamado cada vez que se quiere bloquear/desbloquear a un usuario
	 * del sistema.</p>
	 *
	 *
	 * @param idUser <code>int</code> identificador del usuario.
	 * 
	 * @return <code>TOUser</code> informaci&oacute;n actualizada del usuario
	 * 
	 */
	@Override
	public TOUser unlockUser(int idUser) throws ExceptionControl {
		
		 
		// Abrimos la sesi&oacute;n de Hibernate.
		   System.out.println("Abrimos sesi&oacute;n de Hibernate");
		   
		   Session session = this.sessionFactory.openSession();
			
		   try
		   {
				 // Comprobamos que el usuario est&aacute; registrado 
			   		System.out.println("Comprobamos que el usuario est&aacute; registrado");
					Object objBOUser = existRegister(session, Table.USER, "id", idUser);
					
				 // Vemos si es nulo o no y si es de tipo BOUser
					if ((objBOUser != null) && (objBOUser instanceof BOUser))
					{
						// Comprobamos que el usuario est&aacute; registrado 
				   		System.out.println("Realizamos Cast a BOUser");
				   		// Realizamos el cast del objeto
						BOUser boUser = (BOUser) objBOUser;
						
						System.out.println("Cambiamos su estado actual: " + boUser.getLocked() + "-->" + !boUser.getLocked());
						//Cambiamos su estado
						boUser.setLocked(!boUser.getLocked());
						
						System.out.println("Actualizamos su estador");
						// Actualizamos el estado del usuario.
						session.saveOrUpdate(boUser);
						
						System.out.println("Realizamos Commit");
						// Commit.
						session.flush();

						// Retornamos un objeto TOUser con la informaci&oacute;n actualizada.
						return boUser.getTOUser();
					}
					
					System.out.println("Usuario no existe");
					// Identificador no V&aacute;lido
					throw (new ExceptionControl(ExceptionControl.MSG_USER_ID_INVALID));
			}
			catch (Exception ex)
			{
				System.out.println("Se ha producido una exception");
				// Propagamos la excepci&oacute;n
				throw (new ExceptionControl(ex.getMessage()));
			}
			finally
			{
				System.out.println("Cerramos sesi&oacute;n");
			 // Cerramos la sesi&oacute;n de Hibernate.
				closeSession(session);
			}
	}

	/**
	 * <p>Este M&eacute;todo es llamado cada vez que se quiere modificar  la informaci&oacute;n de
	 * un usuario a partir de la informaci&oacute;n recibida por par&aacute;metro </p>
	 * 
	 * @param idUser <code>int</code> identificador del usuario.
	 * @param login <code>String</code> login del usuario.
	 * @param password <code>String</code> password del usuario.
	 * @param name <code>String</code> nombre real del usuario.
	 * @param company <code>String</code> nombre de la empresa del usuario.
	 * @param email <code>String</code> email del usuario.
	 * @param type <code>String</code> tipo de autorizaci&oacute;n.
	 * @param isLocked <code>boolean</code> indicador de bloqueo
	 * 
	 * @return <code>TOUser</code> informaci&oacute;n del usuario actualizada
	 *  
	 */
	@Override
	public TOUser modifyUser(int idUser, String login, String password,
			String name, String company, String email, String type, boolean locked)
			throws ExceptionControl {
		
			// Abrimos la sesi&oacute;n de Hibernate.
			Session session = this.sessionFactory.openSession();
			
			try
			{
				// Comprobamos que el usuario est&aacute; registrado 
				   Object objBOUser = existRegister(session, Table.USER, "id", idUser);
				   
				// Comprobamos que no es nulo y adem&aacute;s es de tipo BOUser
				   if ((objBOUser != null) && (objBOUser instanceof BOUser))
					{		
						// Realizamos el cast del objeto
						BOUser boUser = (BOUser) objBOUser;
												
						//Al usuario administrador no se le puede cambiar el tipo de Autorizaci&oacute;n
						if (boUser.getType().equals(TypeAutoriz.ADMINISTRADOR.toString()) && !type.equals(TypeAutoriz.ADMINISTRADOR.toString()))
						{
							// No se puede cambiar el rol del &uacute;nico administrador del sistema
							// para que deje de serlo, por lo que lanzamos una BusinessRulesViolationException.
							throw (new ExceptionControl(ExceptionControl.MSG_USER_ADMIN_TYPE));
							
						}
						
						//Comprobamos que no existe el login asignado a otro usuario
						if(!boUser.getLogin().equals(login) && 
								existRegister(session, Table.USER, "login", login) != null){
							//El login se encuentra registrado con otro usuario
							throw (new ExceptionControl(ExceptionControl.MSG_USER_LOGIN_REGISTER));
							
						}
						
						//Comprobamos que no existe el email asignado a otro usuario
						if(!boUser.getEmail().equals(email) && 
								existRegister(session, Table.USER, "email", email) != null){
							//El email se encuentra registrado con otro usuario
							throw (new ExceptionControl(ExceptionControl.MSG_USER_EMAIL_REGISTER));
						}
																		
						// Actualizamos la Informaci&oacute;n del Usuario
						boUser.setLogin(login.trim());
						boUser.setPassword(password.trim());
						boUser.setName(name.trim());
						boUser.setCompany(company.trim());
						boUser.setEmail(email.trim());
						boUser.setType(type.trim());
						boUser.setLocked(locked);
						
						// Actualizamos la informaci&oacute;n del usuario.
						session.saveOrUpdate(boUser);
						// Commit
						session.flush();
						// Retornamos un objeto TOUser con la informaci&oacute;n actualizada.
						return boUser.getTOUser();
					}
				   
				// Identificador no V&aacute;lido
					throw (new ExceptionControl(ExceptionControl.MSG_USER_ID_INVALID));
			}
			catch (Exception ex)
			{
				// Propagamos la excepci&oacute;n
				throw (new ExceptionControl(ex.getMessage()));
			}
			finally
			{
			 // Cerramos la sesi&oacute;n de Hibernate.
				closeSession(session);
			}
	}

	
	/**
	 * <p>Este M&eacute;todo es llamado cada vez que se quiere dar de alta a un usuario
	 * del sistema a partir de la informaci&oacute;n recibida por par&aacute;metro</p>
	 * 
	 * @param idUser <code>int</code> identificador del usuario.
	 * 
	 */
	@Override
	public void deleteUser(int idUser) throws ExceptionControl {
		
		// Abrimos la sesi&oacute;n de Hibernate.
		   Session session = sessionFactory.openSession();
		   
		   try
			{
			// Comprobamos que el usuario est&aacute; registrado 
			   Object objBOUser = existRegister(session, Table.USER, "id", idUser);
			   
			// Comprobamos que no es nulo y adem&aacute;s es de tipo BOUser.
			   if ((objBOUser != null) && (objBOUser instanceof BOUser))
			   {
				 // Realizamos el cast del objeto
				 BOUser boUser = (BOUser) objBOUser;
					  
				 // Comprobamos si se trata de un administrador del sistema.
				 if (boUser.getType().equals(TypeAutoriz.ADMINISTRADOR.toString()))
					{
					// Se trata del Usuario Administrador no permitimos su eliminaci&oacute;n
						throw (new ExceptionControl(ExceptionControl.MSG_USER_ADMIN_ACTION));
					}
				 
				 //Recuperamos todos los usuarios-roles definidos en los Proyectos
				 List<Object> list = getRegisters(session, Table.USERROL, "iduser", boUser.getId());
				   
				 // Recorremos todos los usuarios-roles recuperados.
				 for (Object objBOUserRol : list)
				 {
				   // Comprobamos que el objeto es de tipo BOUserRol.
				   if (objBOUserRol instanceof BOUserRol)
				   {
					   // Realizamos el cast del objeto
					   BOUserRol boUserRol = (BOUserRol) objBOUserRol;
						   
					   session.delete(boUserRol);
				   }
				 }
				 
				// Borramos usuario
				 session.delete(boUser);
				 
				// Commit.
				 session.flush();
					
				}
				else 
				{
				 // Identificador no V&aacute;lido
					throw (new ExceptionControl(ExceptionControl.MSG_USER_ID_INVALID));
				}
			}		
		   catch (Exception ex)
			{
				// Propagamos la excepci&oacute;n
				throw (new ExceptionControl(ex.getMessage()));
			}
			finally
			{
			 // Cerramos la sesi&oacute;n de Hibernate.
				closeSession(session);
			}
		
	}

	/**
	 * <p>Este m&eacute;todo es llamado cada vez que se quiere mandar los datos de registro
	 * a un usuario.</p>
	 * 
	 */
	@Override
	public void emailDatosUser(int idUser) throws ExceptionControl {
			
		// Abrimos la sesi&oacute;n de Hibernate.
		   Session session = sessionFactory.openSession();
		   
		   try
			{
			   
			//Comprobamos que existe el Usuario 
			   Object objBOUser = existRegister(session, Table.USER, "id", idUser);
			   
			// Comprobamos que no es nulo y adem&aacute;s es de tipo BOUser.
			   if ((objBOUser != null) && (objBOUser instanceof BOUser)){
				   
				   // Realizamos el cast del objeto
					 BOUser boUser = (BOUser) objBOUser;
					 
					 
				   // Obtenemos la configuraci&oacute;n del Correo
					 Object objBOConfigHOST = existRegister(session, Table.CONFIG, "name", ParamEmail.HOST.toString());
					 Object objBOConfigPORT = existRegister(session, Table.CONFIG, "name", ParamEmail.PORT.toString());
					 Object objBOConfigTLS = existRegister(session, Table.CONFIG, "name", ParamEmail.TLS.toString());
					 Object objBOConfigAUTH = existRegister(session, Table.CONFIG, "name", ParamEmail.AUTH.toString());
					 Object objBOConfigSOCKETP = existRegister(session, Table.CONFIG, "name", ParamEmail.SOCKET_PORT.toString());
					 Object objBOConfigSOCKETC = existRegister(session, Table.CONFIG, "name", ParamEmail.SOCKET_CLASS.toString());
					 Object objBOConfigUSER = existRegister(session, Table.CONFIG, "name", ParamEmail.USER.toString());
					 Object objBOConfigPASSWD = existRegister(session, Table.CONFIG, "name", ParamEmail.PASSWD.toString());
					 Object objBOConfigFROM = existRegister(session, Table.CONFIG, "name", ParamEmail.FROM.toString());
					   
				   // Comprobamos que no es nulo y las clases a las que pertenecen.
					  if ((objBOConfigHOST != null)    && (objBOConfigHOST instanceof BOConfig)
					   && (objBOConfigPORT != null)    && (objBOConfigPORT instanceof BOConfig)
					   && (objBOConfigTLS != null)     && (objBOConfigTLS instanceof BOConfig)
					   && (objBOConfigAUTH != null)    && (objBOConfigAUTH instanceof BOConfig)
					   && (objBOConfigSOCKETP != null) && (objBOConfigSOCKETP instanceof BOConfig)
					   && (objBOConfigSOCKETC != null) && (objBOConfigSOCKETC instanceof BOConfig)
					   && (objBOConfigUSER != null)    && (objBOConfigUSER instanceof BOConfig)
					   && (objBOConfigPASSWD != null)  && (objBOConfigPASSWD instanceof BOConfig)
					   && (objBOConfigFROM != null)    && (objBOConfigFROM instanceof BOConfig))
						{
		  			    
					     // Realizamos el cast del objeto
							BOConfig configHOST    = (BOConfig) objBOConfigHOST;
							BOConfig configPORT    = (BOConfig) objBOConfigPORT;
							BOConfig configTLS     = (BOConfig) objBOConfigTLS;
							BOConfig configAUTH    = (BOConfig) objBOConfigAUTH;
							BOConfig configSOCKETP = (BOConfig) objBOConfigSOCKETP;
							BOConfig configSOCKETC = (BOConfig) objBOConfigSOCKETC;
							BOConfig configUSER    = (BOConfig) objBOConfigUSER;
							BOConfig configPASSWD  = (BOConfig) objBOConfigPASSWD;
							BOConfig configFROM    = (BOConfig) objBOConfigFROM;
								

								String subjectMensaje = "Datos de Registro Integra";
								
								String bodyMensaje = "Sus datos de registro son: \n\n"
										           + "  Usuario   : " + boUser.getLogin() +"\n"
										           + "  Clave     : " + boUser.getPassword();
								
								
						 // Enviamos el Correo
							edu.uoc.integra.lib.utils.Validations.sendEmail(configHOST.getValue(), configPORT.getValue(), configTLS.getValue(), 
									  configAUTH.getValue(), configSOCKETP.getValue(), configSOCKETC.getValue(), configUSER.getValue(), configPASSWD.getValue(), 
									  configFROM.getValue(), boUser.getEmail(), subjectMensaje, bodyMensaje);
								
						}
				   
			   }else{
				   
				// Identificador no V&aacute;lido
				throw (new ExceptionControl(ExceptionControl.MSG_USER_ID_INVALID));
			   }
			
			}
			catch (Exception ex)
			{
				// Propagamos la excepci&oacute;n
				throw (new ExceptionControl(ex.getMessage()));
			}
			finally
			{
				// Cerramos la sesi&oacute;n de Hibernate.
				closeSession(session);
			}
		
	}


	/**
	 * <p>Este M&eacute;todo es llamado cada vez que se quiere consultar  la informaci&oacute;n de
	 * un usuario a partir de la informaci&oacute;n recibida por par&aacute;metro </p>
	 * 
	 * @param idUser <code>int</code> identificador del usuario.
	 * 
	 * @return <code>TOUser</code> contiene la informaci&oacute;n del usuario cuyo
	 * <i>id</i> es igual al que se ha recibido como par&aacute;metro.
	 * 
	 */
	@Override
	public TOUser getUser(int idUser) throws ExceptionControl {
		

		   System.out.println("Abrimos sesi&oacute;n de Hibernate");
		 // Abrimos la sesi&oacute;n de Hibernate.
			Session session = this.sessionFactory.openSession();
			
			try
			{
				System.out.println("Comprobamos que el usuario est&aacute; registrado");
				// Comprobamos que el usuario est&aacute; registrado
				Object objBOUser = existRegister(session, Table.USER, "id", idUser);
				
				// Comprobamos que no es nulo y adem&aacute;s es de tipo BOUser.
				if ((objBOUser != null) && (objBOUser instanceof BOUser))
				{
					System.out.println("Casteamos a objBOUser");
					 // Realizamos el cast del objeto
					 BOUser boUser = (BOUser) objBOUser;
					
					 System.out.println("Retornamos Datos de Usuario");
					 //Retornamos la informaci&oacute;n del objeto en un objeto TOUser
					return boUser.getTOUser();
					
				}
				
				System.out.println("Usuario no existe");
				 // Identificador no V&aacute;lido
				throw (new ExceptionControl(ExceptionControl.MSG_USER_ID_INVALID));
			}
			finally
			{
				System.out.println("Cerramos sesi&oacute;n");
				// Cerramos la sesi&oacute;n de Hibernate.
				closeSession(session);
			}
	}
	
		
}
