package edu.uoc.integra.business.interfaces;

import java.util.Date;
import java.util.List;

import edu.uoc.integra.business.exceptions.ExceptionControl;
import edu.uoc.integra.business.objects.transfers.TOProject;
import edu.uoc.integra.business.objects.transfers.TORol;
import edu.uoc.integra.business.objects.transfers.TOState;
import edu.uoc.integra.business.objects.transfers.TOUserRol;
import edu.uoc.integra.business.objects.transfers.TOWorkFlow;

/**
 * <p>Interfaz que define todas las funcionalidades que est&aacute;n relacionadas
 * con la gesti&oacute;n de proyectos, roles, estados, definici&oacute;n de workflow. 
 * Estas funcionalidades son presentadas a la la capa de presentaci&oacute;n desde la capa de negocio</p>
 * 
 * @author Miguel Angel Pineda Cruz
 * @version 1.0 
 */
public interface IBProjectManager 
{
	
	/**
	 * <p>Este M&eacute;todo es llamadado cada vez que se quiera dar de alta a un proyecto
	 * a partir de la informaci&oacute;n recibida por par&aacute;metro.</p>
	 * 
	 * @param project <code>TOProject</code> Informaci&oacute;n del Proyecto a crear.
	 * 
	 * @return <code>TOProject</code> contiene la informaci&oacute;n del proyecto creado</p>
	 * 
	 */
	public abstract TOProject createProject(TOProject project)  throws ExceptionControl;
	
	
	/**
	 * <p>Este M&eacute;todo es llamadado cada vez que se quiera dar de alta a un proyecto
	 * a partir de la informaci&oacute;n recibida por par&aacute;metro.</p>
	 * 
	 * @param project <code>TOProject</code> Informaci&oacute;n del Proyecto a crear.
	 * @param idProjectFrom <code>int</code> Informaci&oacute;n del Proyecto del que heredamos las Caracter&iacute;sticas
	 * 
	 * @return <code>TOProject</code> contiene la informaci&oacute;n del proyecto creado</p>
	 * 
	 */
	public abstract TOProject createProjectFrom(TOProject project, int idProjectFrom)  throws ExceptionControl; 
	
	
	
	/**
	 * <p>Este m&eacute;todo es llamado cada vez que se quiere consultar la lista de los
	 * proyectos registrados en el sistema.</p>
	 * 
	 * @return <code>List&lt;TOProject&gt;</code> lista de proyectos
	 * 
	 */
	public abstract List<TOProject> listProject() throws ExceptionControl;
	
	/**
	 * <p>Este m&eacute;todo es llamado cada vez que se quiere consultar la lista de los
	 * proyectos registrados en el sistema que cumplen ciertos criterios, establecidos con la
	 * informaci&oacute;n que se recibe por par&aacute;metro.</p>
	 * 
	 * @return  <code>List&lt;TOProject&gt;</code> lista de proyectos
	 * 
	 */
	public abstract List<TOProject> listProject(String prefix_code, String name) throws ExceptionControl;
	
	/**
	 * <p>Este M&eacute;todo es llamado cada vez que se quiere modificar  la informaci&oacute;n de
	 * un proyecto a partir de la informaci&oacute;n recibida por par&aacute;metro </p>
	 * 
	 * @param idProject <code>int</code> identificador del proyecto.
	 * @param prefix_code <code>String</code> c&oacute;digo del proyecto.
	 * @param name <code>String</code> nombre del proyecto.
	 * @param descript <code>String</code> descripci&oacute;n del proyecto.
	 * @param fecopen <code>Date</code> fecha inicio del proyecto.
	 * @param fecclose <code>Date</code> fecha fin del proyecto. 
	 * 
	 * @return <code>TOProject</code> informaci&oacute;n del proyecto actualizada
	 *  
	 */
	public abstract TOProject modifyProject(int idProject, String prefix_code, String name, String descript, Date fecopen, Date fecclose) throws ExceptionControl; 
		
	/**
	 * <p>Este M&eacute;todo es llamado cada vez que se quiere eliminar a un proyecto
	 * del sistema a partir de la informaci&oacute;n recibida por par&aacute;metro</p>
	 * 
	 * @param idProject <code>int</code> identificador del proyecto.
	 * 
	 */
	public abstract void deleteProject(int idProject) throws ExceptionControl; 
		
	/**
	 * <p>Este M&eacute;todo es llamado cada vez que se quiere consultar  la informaci&oacute;n de
	 * un proyecto a partir de la informaci&oacute;n recibida por par&aacute;metro</p>
	 * 
	 * @param idProject <code>int</code> identificador del proyecto.
	 * 
	 * @return <code>TOProject</code> contiene la informaci&oacute;n del proyecto cuyo
	 * <i>id</i> es igual al que se ha recibido como par&aacute;metro.
	 * 
	 */
	public abstract TOProject getProject(int idProject) throws ExceptionControl;
	
	/**
	 * <p>Este M&eacute;todo es llamadado cada vez que se quiera dar de alta a un rol dentro de un proyecto
	 * a partir de la informaci&oacute;n recibida por par&aacute;metro.</p>
	 * 
	 * @param rol <code>TORol</code> Informaci&oacute;n del Rol de un Projecto.
	 * 
	 * @return <code>TORol</code> contiene la informaci&oacute;n del rol creado</p>
	 * 
	 */
	public abstract TORol createRolProject(TORol rol)  throws ExceptionControl; 
	
	/**
	 * <p>Este m&eacute;todo es llamado cada vez que se quiere consultar la lista de
	 * roles definidos en un proyecto.</p>
	 * 
	 * @param project <code>TOProject</code> Informaci&oacute;n del Projecto.
	 * 
	 * @return <code>List&lt;TORol&gt;</code> lista de roles de un Proyecto
	 * 
	 */
	public abstract List<TORol> listRolProject(TOProject project) throws ExceptionControl;
	
	/**
	 * <p>Este M&eacute;todo es llamado cada vez que se quiere modificar  la informaci&oacute;n de
	 * un rol definido en un proyecto a partir de la informaci&oacute;n recibida por par&aacute;metro </p>
	 * 
	 * @param idProject <code>int</code> identificador del proyecto.
	 * @param idRol <code>int</code> identificador del rol del proyecto.
	 * @param nameRol <code>String</code> nombre del rol.
	 * 
	 * @return <code>TORol</code> informaci&oacute;n del rol actualizada
	 *  
	 */
	public abstract TORol modifyRolProject(int idProject, int idRol, String nameRol) throws ExceptionControl; 
		
	/**
	 * <p>Este M&eacute;todo es llamado cada vez que se quiere eliminar a un rol definido en un proyecto
	 * a partir de la informaci&oacute;n recibida por par&aacute;metro</p>
	 * 
	 * @param idProject <code>int</code> identificador del proyecto. 
	 * @param idRol <code>int</code> identificador del rol.
	 * 
	 */
	public abstract void deleteRolProject(int idProject, int idRol) throws ExceptionControl; 
		
	/**
	 * <p>Este M&eacute;todo es llamado cada vez que se quiere consultar  la informaci&oacute;n de
	 * un rol definido en un proyecto a partir de la informaci&oacute;n recibida por par&aacute;metro</p>
	 * 
	 * @param idProject <code>int</code> identificador del proyecto.
	 * 
	 * @return <code>TORol</code> contiene la informaci&oacute;n del rol cuyo
	 * <i>idRol</i> es igual al que se ha recibido como par&aacute;metro.
	 * 
	 */
	public abstract TORol getRolProject(int idProject, int idRol) throws ExceptionControl;

	/**
	 * <p>Este M&eacute;todo es llamadado cada vez que se quiera dar de alta a un estado dentro de un proyecto
	 * a partir de la informaci&oacute;n recibida por par&aacute;metro.</p>
	 * 
	 * @param state <code>TOState</code> Informaci&oacute;n del estado a crear.
	 * 
	 * @return <code>TOState</code> contiene la informaci&oacute;n del estado creado</p>
	 * 
	 */
	public abstract TOState createStateProject(TOState state)  throws ExceptionControl; 
	
	/**
	 * <p>Este m&eacute;todo es llamado cada vez que se quiere consultar la lista de
	 * estados definidos en un proyecto.</p>
	 * 
	 * @param project <code>TOProject</code> Informaci&oacute;n del proyecto.
	 * 
	 * @return <code>List&lt;TOState&gt;</code> lista de estados de un Proyecto
	 * 
	 */
	public abstract List<TOState> listStateProject(TOProject project) throws ExceptionControl;
	
	/**
	 * <p>Este M&eacute;todo es llamado cada vez que se quiere modificar  la informaci&oacute;n de
	 * un estado definido en un proyecto a partir de la informaci&oacute;n recibida por par&aacute;metro </p>
	 * 
	 * @param idProject <code>int</code> identificador del proyecto.
	 * @param idState <code>int</code> identificador del estado del proyecto.
	 * @param nameState <code>String</code> nombre del estado.
	 * @param descriptState <code>String</code> descripci&oacute;n del estado.
	 * 
	 * @return <code>TOState</code> informaci&oacute;n del estado actualizada
	 *  
	 */
	public abstract TOState modifyStateProject(int idProject, int idState, String nameState) throws ExceptionControl; 
		
	/**
	 * <p>Este M&eacute;todo es llamado cada vez que se quiere eliminar a un estado definido en un proyecto
	 * a partir de la informaci&oacute;n recibida por par&aacute;metro</p>
	 * 
	 * @param idProject <code>int</code> identificador del proyecto. 
	 * @param idState <code>int</code> identificador del estado.
	 * 
	 */
	public abstract void deleteStateProject(int idProject, int idState) throws ExceptionControl; 
		
	/**
	 * <p>Este M&eacute;todo es llamado cada vez que se quiere consultar  la informaci&oacute;n de
	 * un estado definido en un proyecto a partir de la informaci&oacute;n recibida por par&aacute;metro</p>
	 * 
	 * @param idProject <code>int</code> identificador del proyecto.
	 * @param idState <code>int</code> identificador del estado.	  
	 * 
	 * @return <code>TOState</code> contiene la informaci&oacute;n del estado cuyo
	 * <i>id</i> es igual al que se ha recibido como par&aacute;metro.
	 * 
	 */
	public abstract TOState getStateProject(int idProject, int idState) throws ExceptionControl;

	/**
	 * <p>Este M&eacute;todo es llamadado cada vez que se quiera dar de alta a un WorkFlow
	 * dentro de un proyecto a partir de la informaci&oacute;n recibida por par&aacute;metro.</p>
	 * 
	 * @param workflow <code>TOWorkFlow</code> Informaci&oacute;n del workflow.
	 * 
	 * @return <code>TOWorkFlow</code> contiene la informaci&oacute;n del workflow creado</p>
	 * 
	 */
	public abstract TOWorkFlow createWorkFlowProject(TOWorkFlow workflow)  throws ExceptionControl; 
	
	/**
	 * <p>Este M&eacute;todo es llamadado cada vez que se quiera dar de alta a un estado dentro de un WorkFlow
	 * de un proyecto a partir de la informaci&oacute;n recibida por par&aacute;metro.</p>
	 * 
	 * @param workflow <code>TOWorkFlow</code> Informaci&oacute;n del workflow. 
	 * @param state <code>TOState</code> Informaci&oacute;n del estado nuevo.
	 * 
	 * @return <code>TOWorkFlow</code> contiene la informaci&oacute;n del workflow actualizado con el nuevo estado</p>
	 * 
	 */
	public abstract TOWorkFlow createWorkFlowStateProject(TOWorkFlow workflow, TOState state)  throws ExceptionControl; 
	
	/**
	 * <p>Este m&eacute;todo es llamado cada vez que se quiere consultar la lista de los
	 * workflows definidos en un proyecto.</p>
	 * 
	 * @param project <code>TOProject</code> Informaci&oacute;n del projecto.
	 * 
	 * @return <code>List&lt;TOWorkFlow&gt;</code> lista de worksflows de un Proyecto
	 * 
	 */
	public abstract List<TOWorkFlow> listWorkFlowProject(TOProject project) throws ExceptionControl;
	
	/**
	 * <p>Este M&eacute;todo es llamado cada vez que se quiere modificar  la informaci&oacute;n de
	 * un workflow definido en un proyecto a partir de la informaci&oacute;n recibida por par&aacute;metro </p>
	 * 
	 * @param workflow <code>TOWorkFlow</code> Informaci&oacute;n del workflow.
	 * 
	 * @return <code>TOWorkFlow</code> informaci&oacute;n del workflow actualizada.
	 *  
	 */
	public abstract TOWorkFlow modifyWorkFlowProject(TOWorkFlow workflow) throws ExceptionControl; 
	
	/**
	 * <p>Este M&eacute;todo es llamado cada vez que se quiere eliminar a un workflow definido en un proyecto
	 * a partir de la informaci&oacute;n recibida por par&aacute;metro</p>
	 * 
	 * @param idProject <code>int</code> identificador del proyecto. 
	 * @param idWorkFlow <code>int</code> identificador del workflow.
	 * 
	 */
	public abstract void deleteWorkFlowProject(int idProject, int idWorkFlow) throws ExceptionControl; 
	
	/**
	 * <p>Este M&eacute;todo es llamado cada vez que se quiere eliminar a un estado de un workflow definido en un proyecto
	 * a partir de la informaci&oacute;n recibida por par&aacute;metro</p>
	 * 
	 * @param idProject <code>int</code> identificador del proyecto. 
	 * @param idWorkFlow <code>int</code> identificador del workflow.
	 * @param idWorkFlowState <code>int</code> identificador de un estado del workflow.
	 * 
	 */
	public abstract void deleteWorkFlowProject(int idProject, int idWorkFlow, int idWorkFlowState) throws ExceptionControl; 
	
	/**
	 * <p>Este M&eacute;todo es llamado cada vez que se quiere consultar  la informaci&oacute;n de
	 * un workflow definido en un proyecto a partir de la informaci&oacute;n recibida por par&aacute;metro</p>
	 * 
	 * @param idProject <code>int</code> identificador del proyecto.
	 * @param idWorkFlow <code>int</code> identificador del workflow.	  
	 * 
	 * @return <code>TOWorkFlow</code> contiene la informaci&oacute;n del workflow cuyo
	 * <i>id</i> es igual al que se ha recibido como par&aacute;metro.
	 * 
	 */
	public abstract TOWorkFlow getWorkFlowProject(int idProject, int idWorkFlow) throws ExceptionControl;
	
	/**
	 * <p>Este M&eacute;todo es llamadado cada vez que se quiera dar de alta una dupla usuario-rol
	 * dentro de un proyecto a partir de la informaci&oacute;n recibida por par&aacute;metro.</p>
	 * 
	 * @param userrol <code>TOUserRol</code> Informaci&oacute;n del user-rol.
	 * 
	 * @return <code>TOUserRol</code> contiene la informaci&oacute;n del user-rol creado</p>
	 * 
	 */
	public abstract TOUserRol createUserRolProject(TOUserRol userrol)  throws ExceptionControl; 
	
	/**
	 * <p>Este m&eacute;todo es llamado cada vez que se quiere consultar la lista de las duplas usuarios-rol
	 * definidos en un proyecto.</p>
	 * 
	 * @param project <code>TOProject</code> Informaci&oacute;n del projecto.
	 * 
	 * @return <code>List&lt;TOUserRol&gt;</code> lista de duplas usuarios-rol de un Proyecto
	 * 
	 */
	public abstract List<TOUserRol> listUserRolProject(TOProject project) throws ExceptionControl;
		
	/**
	 * <p>Este M&eacute;todo es llamado cada vez que se quiere eliminar una dupla usuario-rol en un proyecto
	 * a partir de la informaci&oacute;n recibida por par&aacute;metro</p>
	 * 
	 * @param idProject <code>int</code> identificador del proyecto. 
	 * @param idUserRol <code>int</code> identificador de la dupla usuario-rol.
	 * 
	 */
	public abstract void deleteUserRolProject(int idProject, int idUserRol) throws ExceptionControl; 
		
	/**
	 * <p>Este M&eacute;todo es llamado cada vez que se quiere consultar  la informaci&oacute;n de
	 * una dupla usuario-rol definida en un proyecto a partir de la informaci&oacute;n recibida por par&aacute;metro</p>
	 * 
	 * @param idProject <code>int</code> identificador del proyecto.
	 * @param idUserRol <code>int</code> identificador de la dupla usuario-rol.	  
	 * 
	 * @return <code>TOUserRol</code> contiene la informaci&oacute;n de la dupla usuario-rol cuyo
	 * <i>id</i> es igual al que se ha recibido como par&aacute;metro.
	 * 
	 */
	public abstract TOUserRol getUserRolProject(int idProject, int idUserR) throws ExceptionControl;
}
