package edu.uoc.integra.business.interfaces;

import java.util.List;

import edu.uoc.integra.business.exceptions.ExceptionControl;
import edu.uoc.integra.business.objects.transfers.TOUser;


/**
 * <p>Interfaz que define todas las funcionalidades que est&aacute;n relacionadas
 * con la gesti&oacute;n de usuarios. Estas funcionalidades son presentadas a la
 * la capa de presentaci&oacute;n desde la capa de negocio</p>
 * 
 * @author Miguel Angel Pineda Cruz
 * @version 1.0 
 */
public interface IBUserManager 
{
	
	/**
	 * <p>Este M&eacute;todo es llamadado cada vez que se quiera dar de alta a un usuario
	 * a partir de la informaci&oacute;n recibida por par&aacute;metro.</p>
	 * 
	 * @param user <code>TOUser</code> Informaci&oacute;n del Usuario a crear.
	 * 
	 * @return <code>TOUser</code> contiene la informaci&oacute;n del usuario creado</p>
	 * 
	 */
	public abstract TOUser createUser(TOUser user)  throws ExceptionControl; 
	
	/**
	 * <p>Este m&eacute;todo es llamado cada vez que se quiere consultar la lista de
	 * usuarios registrados en el sistema.</p>
	 * 
	 * @return <code>List&lt;TOUser&gt;</code> lista de usuarios
	 * 
	 */
	public abstract List<TOUser> listUser() throws ExceptionControl;
	
	/**
	 * <p>Este m&eacute;todo es llamado cada vez que se quiere consultar la lista de
	 * usuarios registrados en el sistema que cumplen ciertos criterios, establecidos con la
	 * informaci&oacute;n que se recibe por par&aacute;metro.</p>
	 * 
	 * @return <code>List&lt;TOUser&gt;</code> lista de usuarios
	 * 
	 */
	public abstract List<TOUser> listUser(String login, String company, String email, String type) throws ExceptionControl;
	
	/**
	 * <p>Este M&eacute;todo es llamado cada vez que se quiere bloquear/desbloquear a un usuario
	 * del sistema.</p>
	 *
	 *
	 * @param idUser <code>int</code> identificador del usuario.
	 * 
	 * @return <code>TOUser</code> informaci&oacute;n actualizada del usuario
	 * 
	 */
	public abstract TOUser unlockUser(int idUser) throws ExceptionControl; 
			
	/**
	 * <p>Este M&eacute;todo es llamado cada vez que se quiere modificar  la informaci&oacute;n de
	 * un usuario a partir de la informaci&oacute;n recibida por par&aacute;metro </p>
	 * 
	 * @param idUser <code>int</code> identificador del usuario.
	 * @param login <code>String</code> login del usuario.
	 * @param password <code>String</code> password del usuario.
	 * @param name <code>String</code> nombre real del usuario.
	 * @param company <code>String</code> nombre de la empresa del usuario.
	 * @param email <code>String</code> email del usuario.
	 * @param type <code>String</code> tipo de autorizaci&oacute;n.
	 * @param isLocked 
	 * 
	 * @return <code>TOUser</code> informaci&oacute;n del usuario actualizada
	 *  
	 */
	public abstract TOUser modifyUser(int idUser, String login, String password, String name, String company, String email, String type, boolean locked) throws ExceptionControl; 
		
	/**
	 * <p>Este M&eacute;todo es llamado cada vez que se quiereeliminar a un usuario
	 * del sistema a partir de la informaci&oacute;n recibida por par&aacute;metro</p>
	 * 
	 * @param idUser <code>int</code> identificador del usuario.
	 * 
	 */
	public abstract void deleteUser(int idUser) throws ExceptionControl; 
		
	/**
	 * <p>Este m&eacute;todo es llamado cada vez que se quiere mandar los datos de registro
	 * al usuario.</p>
	 * 
	 * @return <code>List<TOUser></code> lista de usuarios
	 * 
	 */
	public abstract void emailDatosUser(int idUser) throws ExceptionControl;
		
	/**
	 * <p>Este M&eacute;todo es llamado cada vez que se quiere consultar  la informaci&oacute;n de
	 * un usuario a partir de la informaci&oacute;n recibida por par&aacute;metro </p>
	 * 
	 * @param idUser <code>int</code> identificador del usuario.
	 * 
	 * @return <code>TOUser</code> contiene la informaci&oacute;n del usuario cuyo
	 * <i>id</i> es igual al que se ha recibido como par&aacute;metro.
	 * 
	 */
	public abstract TOUser getUser(int idUser) throws ExceptionControl;
	
	
}
