package edu.uoc.integra.lib.utils;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Locale;
import java.util.Properties;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import javax.mail.Message;
import javax.mail.MessagingException;
import javax.mail.PasswordAuthentication;
import javax.mail.Session;
import javax.mail.Transport;
import javax.mail.internet.InternetAddress;
import javax.mail.internet.MimeMessage;

/**
 * <p>Clase que implementa diversos m&eacute;todos de validaci&oacute;n</p>
 * 
 * @author Miguel Angel Pineda Cruz
 * @version 1.0
 */
public class Validations
{
	
	// -------- CONSTANTES.
	//Par&aacute;metros de envio de Correo
	private static final String SMTP_HOST = "mail.smtp.host";
	private static final String SMTP_PORT = "mail.smtp.port";
	private static final String SMTP_TLS  = "mail.smtp.starttls.enable";
	private static final String SMTP_AUTH = "mail.smtp.auth";
	private static final String SMTP_SOCKET_PORT  = "mail.smtp.socketFactory.port";
	private static final String SMTP_SOCKET_CLASS = "mail.smtp.socketFactory.class";
	
	
	// ---------- METODOS PUBLICOS Y ESTATICOS
	/**
	 * <p>Comprueba si el <code>String</code> que recibe como par&aacute;metro est&aacute;
	 * vac&iacute;o o no. Se considera que un <code>String</code> est&aacute; vac&iacute;o
	 * si es <code>null</code> o su longitud (n&uacute;mero de caracteres que lo forman),
	 * tras ignorar los espacios en blanco que pueda contener al principio o final, es
	 * igual a cero.</p>
	 * 
	 * @param param <code>String</code> que se quiere verificar.
	 * 
	 * @return <code>true</code> si se cumplen las condiciones en el <code>String</code> para que
	 * sea considerado como vac&iacute;o o <code>false</code> en caso contrario.
	 */
	public static boolean isEmpty(String param)
	{
		// Si el parametro que se recibe es null se retorna true.
		if (param == null) return true;
		// Comprobamos que la longitud de la cadena no sea cero.
		if (param.trim().length() == 0) return true;
		// Si llegamos a este punto quiere decir que la cadena no estar&aacute; vac&iacute;a.
		return false;
	}
	
	/**
	 * <p>Comprueba si el <code>String</code> que se le pasa como par&aacute;metro 
	 * est&aacute; formado por un  n&uacute;mero de caracteres comprendidos entre el 
	 * valor <i>min</i> y <i>max</i> sin tener en cuenta los espacios en blanco que 
	 * el <code>String</code> pueda poseer al principio o final.</p>
	 * 
	 * @param param <code>String</code> que se quiere verificar.
	 * @param min <code>int</code> n&uacute;mero m&iacute;nimo de caracteres.
	 * @param max <code>int</code> n&uacute;mero m&aacute;ximo de caracteres.
	 * @return <code>true</code> en el caso de que el <code>String</code> tenga un 
	 * n&uacute;mero de caracteres dentro del rango que se ha pasado como par&aacute;metro 
	 * o <code>false</code> en caso contrario.
	 */
	public static boolean isInRange(String param, int min, int max)
	{
		// Comprobamos que min y max son valores mayores que cero. Si no es as&iacute; retornamos false.
		if ((min < 0) || (max < 0)) return false;
		// Comprobamos que min <= max. Si no es as&iacute; retornamos false.
		if (min > max) return false;
		// Si el String que se ha pasado es vac&iacute;o retornaremos false.
		if (isEmpty(param)) return false;
		// Comprobamos si el n&uacute;mero de caracteres de la cadena cae en el rango especificado.
		if ((param.trim().length() >= min) && (param.trim().length() <= max))
			// Cae en el rango as&iacute; que retornamos true.
			return true;
		// Aqu&iacute; solamente llegaremos si el n&uacute;mero de caracteres no cae en el rango.
		return false;
	}
	
	/**
	 * <p>Comprueba si el <code>String</code> que se le pasa como par&aacute;metro contiene 
	 * una direcci&oacute;n de correo electr&oacute;nico v&aacute;lida. El m&eacute;todo no comprueba la existencia
	 * de dicha direcci&oacute;n de correo, solamente su formato.</p>
	 * 
	 * @param param <code>String</code> que se quiere verificar.
	 * @return <code>true</code> si <i>param</i> contiene una direcci&oacute;n de correo
	 * electr&oacute;nico v&aacute;lida o <code>false</code> en caso contrario.
	 */
	public static boolean isEmail(String param)
	{
		// Si "param" es null retornamos false.
		if (param == null) return false;
		// Utilizamos una expresi&oacute;n regular para verificar el email.
		Pattern patter = Pattern.compile("[a-zA-Z0-9]+[.[a-zA-Z0-9_-]+]*@[a-z0-9][\\w\\.-]*[a-z0-9]\\.[a-z][a-z\\.]*[a-z]$");
		Matcher matcher = patter.matcher(param.trim());
		// Vemos si se ha producido una coincidencia o no.
		if (matcher.find()) return true;
		// Llegados a este punto no se habr&aacute; producido una coincidencia.
		return false;
	}
	
	/**
	 * <p>Comprueba que los valores pasados en los par&aaacute;metros se corresponden con los de una
	 * fecha v&aacute;lida.</p>
	 * 
	 * @param day   <code>String</code> d&iacute;a 
	 * @param month <code>String</code> mes.
	 * @param year  <code>String</code> a&ntilde;o
	 *
	 * @return <code>true</code> si la fecha recibida por par&aacute;metro es una fecha v&aacute;lida o <code>false</code> en caso contrario.
	 */
	public static boolean isValidDate(String day, String month, String year)
	{
		// Si alguno de los par&aacute;metros es null retornamos false.
		if ((day == null || month == null || year == null)) return false;
		
		// Utilizamos un objeto SimpleDateFormat.
		try 
		{
			SimpleDateFormat sdf = new SimpleDateFormat("dd/MM/yyyy", Locale.getDefault());
			
			// Establecemos bisiestos.
			sdf.setLenient(false);
			
			// Interpretamos la fecha pasada.
			sdf.parse(day + "/" + month + "/" + year);
			
			// La fecha es correcta.
			return true;
		}
		catch (ParseException pe)
		{
			// Los par&aacute;metros no corresponden con una fecha.
			return false;
		}
	}
	
	/**
	 * <p>Comprueba que los valores pasados en los par&aaacute;metros se corresponden con los de una
	 * fecha v&aacute;lida.</p>
	 * 
	 * @param day <code>String</code> d&iacute;a.
	 * @param month <code>String</code> mes.
	 * @param year <code>String</code> a&ntilde;o
	 * @param hour <code>String</code> hora 
	 * @param minutes <code>String</code> minutos 
	 * @return <code>true</code> si la fecha recibida por par&aacute;metro es una fecha v&aacute;lida o <code>false</code> en caso contrario.
	 */
	public static boolean isValidDate(String day, String month, String year, String hour, String minutes)
	{
		// Comprobamos que los par&aacute;metros no sean nulos.
		if (isEmpty(day)) return false;
		if (isEmpty(month)) return false;
		if (isEmpty(year)) return false;
		if (isEmpty(hour)) return false;
		if (isEmpty(minutes)) return false;
		
		// Preparamos la cadena que formar&aacute; la fecha.
		String date = day + "-" + month + "-" + year + " " + hour + ":" + minutes;
		
		// Creamos un SimpleDateFormat con el patr&oacute;n que vamos a seguir.
		SimpleDateFormat sdf = new SimpleDateFormat("dd-MM-yyyy HH:mm");
		
		// Establecemos que interprete las fechas estrictamente.
		sdf.setLenient(false);
		
		try
		{
			// Intentamos crear un objeto Date a partir de la informaci&oacute;n que tenemos.
			sdf.parse(date);
			
			// Si llegamos a este punto la fecha ser&aacute; correcta.
			return true;
		}
		catch (ParseException pe)
		{
			// Se ha producido una excepci&oacute;n debido a que la fecha no es correcta.
			return false;
		}
	}
	
	/**
	 * <p>Comprueba si el par&aacute;metro que recibe puede ser interpretado como un n&uacute;mero entero
	 * o no.</p>
	 * 
	 * @param number <code>String</code> a evaluar.
	 * @return <code>true</code> si el par&aacute;metro puede ser evaluado como un entero o <code>false</code>
	 * en caso contrario.
	 */
	public static boolean isInteger(String number)
	{
		// Comprobamos si es nulo.
		if (isEmpty(number)) return false;
		try
		{
			// Intentamos interpretar el string como un entero.
			Integer.parseInt(number);
			
			// Si llegamos a este punto querr&aacute; decir que es un entero.
			return true;
		}
		catch (NumberFormatException nfe)
		{
			// Se lanza una excepci&oacute;n porque el par&aacute;metro no puede ser interpretado como un n&uacute;mero.
			return false;
		}
	}
	
	/**
	 * <p>Comprueba si el par&aacute;metro que se recibe puede ser interpretado como un n&uacute;mero
	 * entero mayor o igual que cero.</p>
	 * 
	 * @param number <code>String</code> a evaluar.
	 * @return <code>true</code> si el par&aacute;metro puede ser como un entero mayor o igual
	 * a certo o <code>false</code> en caso contrario.
	 */
	public static boolean isIntegerPositive(String number)
	{
		// Comprobamos si el par&aacute;metro es un entero.
		if (isInteger(number))
		{
			// Retornamos el resultado de compararlo con 0.
			return ((Integer.parseInt(number)) >= 0);
		}
		else
		{
			// No se puede interpretar como un entero.
			return false;
		}
	}
	
	/**
	 * <p>Comprueba si el par&aacute;metro que recibe puede ser interpretado como un n&uacute;mero double
	 * o no.</p>
	 * 
	 * @param number <code>String</code> a evaluar.
	 * @return <code>true</code> si el par&aacute;metro puede ser evaluado como un entero o <code>false</code>
	 * en caso contrario.
	 */
	public static boolean isDouble(String number)
	{
		// Comprobamos si es nulo.
		if (isEmpty(number)) return false;
		try
		{
			// Intentamos interpretar el string como un entero.
			Double.parseDouble(number);
			
			// Si llegamos a este punto querr&aacute; decir que es un entero.
			return true;
		}
		catch (NumberFormatException nfe)
		{
			// Se lanza una excepci&oacute;n porque el par&aacute;metro no puede ser interpretado como un n&uacute;mero.
			return false;
		}
	}
	
	/**
	 * <p>Envio de Correo a partir de los par&aacute;metros recibidos.</p>
	 * 
	 * @param host <code>String</code> servidor de correo SMTP.
	 * @param port <code>int</code> puerto SMTP en el servidor de correo.
	 * @param tls <code>boolean</code> indica si el servidor utiliza TLS (Transport Layer Security).
	 * @param auth <code>boolean</code> indica si es necesario autentificarse en el servidor SMTP.
	 * @param socket_port <code>int</code> puerto de socket SMTP en el servidor de correo.
	 * @param socket_class <code>String</code> class de socket SMTP en el servidor de correo.
	 * @param user <code>String</code> usuario del servidor SMTP.
	 * @param password <code>String</code> contrase&ntilde;a del usuario del servidor SMTP. 
	 * @param from <code>String</code> direcci&oacute;n de correo electr&oacute;nico de la persona que env&iacute;a el email.
	 * @param to <code>String</code> direcci&oacute;n de correo electr&oacute;nico de la persona que va a recibir el email.
	 * @param subject <code>String</code> asunto del correo.
	 * @param body <code>String</code> cuerpo del correo.
	 * 
	 * @throws Exception si se produce un error a la hora de enviar el correo electr&oacute;nico.
	 */
	public static void sendEmail(String host, String port, String tls, String auth, String socket_port,
			String socket_class, final String user,	final String password, String from, String to, String subject, String body) throws Exception
	{
		
		try
		{
			
			System.out.println("Ejecutamos proceso de envio de correo...");
			System.out.println(SMTP_HOST + "=" + host.trim() + "#");
			System.out.println(SMTP_PORT + "=" + port.trim() + "#");
			System.out.println(SMTP_TLS  + "=" + tls.trim() + "#");
			System.out.println(SMTP_AUTH + "=" + auth.trim() + "#");
			System.out.println(SMTP_SOCKET_PORT + "=" + socket_port.trim() + "#");
			System.out.println(SMTP_SOCKET_CLASS + "=" + socket_class.trim() + "#");
			System.out.println("USER=" + user + "#");
			System.out.println("USER=" + password + "#");
			System.out.println("from=" + from + "#");
			System.out.println("to=" + to + "#");
			System.out.println("subject=" + subject + "#");
			System.out.println("body=" + subject + "body");
			
			// Creamos un objeto Properties.
			Properties properties = new Properties();
			//Establecemos el servidor SMTP.
			properties.setProperty(SMTP_HOST, host.trim());
			//Establecemos el puerto del servidor SMTP.
			properties.setProperty(SMTP_PORT, port.trim());
			// Establecemos si se utiliza TLS.
			properties.setProperty(SMTP_TLS, tls.trim());
			//Establecemos si se es necesaria la autentificaci&oacute;n SMTP.
			properties.setProperty(SMTP_AUTH, auth.trim());
			//Establecemos SOCKET PORT
			properties.setProperty(SMTP_SOCKET_PORT, socket_port.trim());
			//Establecemos SOCKET CLASS
			properties.setProperty(SMTP_SOCKET_CLASS, socket_class.trim());
			
			// Creamos una sesi&oacute;n de JavaMail en el servidor
			Session session = Session.getDefaultInstance(properties, 
					// Creamos una Authenticar con los datos del usuario y la contrase&ntilde;a.
					new javax.mail.Authenticator() 
					    {
							// Sobreescribimos el m&eacute;todo.
				   			protected PasswordAuthentication getPasswordAuthentication() 
				   			{
				   				return new PasswordAuthentication(user, password);
				   			}
					    });
			
			// Creamos un objeto para contener el mensaje.
			MimeMessage message = new MimeMessage(session);
			
			// Informaci&oacute;n de quien envia
			message.setFrom(new InternetAddress(from));
			// Destinatario del mensaje.
			message.addRecipient(Message.RecipientType.TO, new InternetAddress(to));
			// Asunto del mensaje.
			message.setSubject(subject, "ISO-8859-1");
			// Cuerpo del mensaje.
			message.setText(body, "ISO-8859-1", "plain");
			
			//Preparamos el mensaje para se lanzado desde un hilo
			final Message msg = message;
 
			new Thread() 
			{
				public void run()
				{
					try
					{
						// Enviamos el mensaje.		
						Transport.send(msg);
					} 
					catch (MessagingException e)
					{
						e.printStackTrace();
					}
				}
			}.start(); //Iniciamos hilo
		}
		catch (Exception ex)
		{
			// propagamos excepci&oacute;n				
			throw ex;
		}
	}
	
}
